#include "ebyte_e07x.h"

/// ʶģ
#if defined(EBYTE_E07_400M10S)
static uint8e_t E07x_NameString[] = "E07-400M10S";
#elif defined(EBYTE_E07_900M10S)
static uint8e_t E07x_NameString[] = "E07-900M10S";
#endif 

/// ʶ汾 
#define EBYTE_E07_TYPE_PROGRAM   0x10 

#define EBYTE_E07X_XOSC         26000000  //26MHz
#define EBYTE_E07X_POW_2_28     268435456 //228η
#define EBYTE_E07X_POW_2_17     131072
#define EBYTE_E07X_CAL_BASE     ((double)EBYTE_E07X_POW_2_28) / ((double)EBYTE_E07X_XOSC)

///ָڲFIFO
#define  FIFO_READ           0x00
#define  FIFO_WRITE          0x01   

///ָж
#define IRQ_RX_SYNC_RECV 0x06  
#define IRQ_TX_SYNC_SEND 0x06  
#define IRQ_RX_CRC_OK    0x07

///ʱ 
static uint8e_t E07x_PaTabel[] = { 0xC0, 0, 0, 0, 0, 0, 0, 0};

///ջ
static uint8e_t E07x_RxBuffer[64];

typedef enum 
{
    GO_INIT          =0x00,        
    GO_BUSY          =0x01, 
    GO_STBY          =0x02,
    GO_RECEIVE       =0x03,
    GO_WAIT_RECEIVE  =0x04,       
    GO_TRANSMIT      =0x05,
    GO_WAIT_TRANSMIT =0x06,
    GO_SLEEP         =0x07,
    GO_ERROR         =0x08
}E07x_Status_t;

/// ״̬ʶ
static E07x_Status_t E07x_Status = GO_INIT;

/* ò 
 * TI SmartRF Studio 7 
 * 汾 2.21.0 */
typedef struct {
    uint8e_t iocfg0;     // GDO0 Output Pin Configuration
    uint8e_t fifothr;    // RX FIFO and TX FIFO Thresholds
    uint8e_t sync1;      // Sync Word, High Byte
    uint8e_t sync0;      // Sync Word, Low Byte
    uint8e_t pktlen;     // Packet Length
    uint8e_t pktctrl0;   // Packet Automation Control
    uint8e_t fsctrl1;    // Frequency Synthesizer Control
    uint8e_t freq2;      // Frequency Control Word, High Byte
    uint8e_t freq1;      // Frequency Control Word, Middle Byte
    uint8e_t freq0;      // Frequency Control Word, Low Byte
    uint8e_t mdmcfg4;    // Modem Configuration
    uint8e_t mdmcfg3;    // Modem Configuration
    uint8e_t mdmcfg2;    // Modem Configuration
    uint8e_t mdmcfg1;    // Modem Configuration
    uint8e_t deviatn;    // Modem Deviation Setting
    uint8e_t mcsm0;      // Main Radio Control State Machine Configuration
    uint8e_t foccfg;     // Frequency Offset Compensation Configuration
    uint8e_t agcctrl2;   // AGC Control
    uint8e_t agcctrl1;   // AGC Control
    uint8e_t worctrl;    // Wake On Radio Control
    uint8e_t fscal3;     // Frequency Synthesizer Calibration
    uint8e_t fscal2;     // Frequency Synthesizer Calibration
    uint8e_t fscal1;     // Frequency Synthesizer Calibration
    uint8e_t fscal0;     // Frequency Synthesizer Calibration
    uint8e_t test2;      // Various Test Settings
    uint8e_t test1;      // Various Test Settings
    uint8e_t test0;      // Various Test Settings
} RF_SETTINGS;

static RF_SETTINGS E07x_InitSetting = {
    0x06,  // IOCFG0        GDO0 Output Pin Configuration
    0x47,  // FIFOTHR       RX FIFO and TX FIFO Thresholds
    0x7A,  // SYNC1         Sync Word, High Byte
    0x0E,  // SYNC0         Sync Word, Low Byte
    0x14,  // PKTLEN        Packet Length
    0x05,  // PKTCTRL0      Packet Automation Control
    0x06,  // FSCTRL1       Frequency Synthesizer Control
    0x10,  // FREQ2         Frequency Control Word, High Byte
    0xB1,  // FREQ1         Frequency Control Word, Middle Byte
    0x3B,  // FREQ0         Frequency Control Word, Low Byte
    0xF5,  // MDMCFG4       Modem Configuration
    0x83,  // MDMCFG3       Modem Configuration
    0x13,  // MDMCFG2       Modem Configuration
    0x02,  // MDMCFG1       Modem Configuration
    0x31,  // DEVIATN       Modem Deviation Setting
    0x18,  // MCSM0         Main Radio Control State Machine Configuration
    0x16,  // FOCCFG        Frequency Offset Compensation Configuration
    0x43,  // AGCCTRL2      AGC Control
    0x49,  // AGCCTRL1      AGC Control
    0xFB,  // WORCTRL       Wake On Radio Control
    0xE9,  // FSCAL3        Frequency Synthesizer Calibration
    0x2A,  // FSCAL2        Frequency Synthesizer Calibration
    0x00,  // FSCAL1        Frequency Synthesizer Calibration
    0x1F,  // FSCAL0        Frequency Synthesizer Calibration
    0x81,  // TEST2         Various Test Settings
    0x35,  // TEST1         Various Test Settings
    0x09,  // TEST0         Various Test Settings
};


/*!
 * @brief ڲĴֵ
 * 
 * @param address Ĵַ
 * @param data    дֵ
 */
static void E07x_SetRegister( uint8e_t address, uint8e_t data )
{
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(0);
    
    /* дַ */
    Ebyte_Port_SpiTransmitAndReceivce( address );
    
    /* дֵ */
    Ebyte_Port_SpiTransmitAndReceivce( data );
    
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(1);    
}

/*!
 * @brief ȡڲĴֵ
 * 
 * @param address Ĵַ
 * @return Ĵֵ
 */
static uint8e_t E07x_GetRegister( uint8e_t address )
{
    uint8e_t result;
    
 
    
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(0);
    
    /* 0x30ַԺļĴ ȡָҪı */
    if( address < 0x30 )
    {
        /* дַ  bit71 ʾҪȡ |0x80 */
        Ebyte_Port_SpiTransmitAndReceivce( address | 0x80);    
    }
    else
    {
         /* дַ  */
        Ebyte_Port_SpiTransmitAndReceivce( address | 0xC0);         
    }

    /* ȡ 1Byte */
    result = Ebyte_Port_SpiTransmitAndReceivce( 0xFF );
    
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(1);  
    
    return result;
}

/*!
 * @brief ڲĴֵ
 * 
 * @param address Ĵַ
 * @param data    ָд׵ַ
 * @param size    д볤
 */
static void E07x_SetRegisters( uint8e_t address , uint8e_t *data , uint8e_t size )
{
    uint8e_t i = size;
    
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(0);
    
    /* дʼַ ַbit6λдָ |0x40 */
    Ebyte_Port_SpiTransmitAndReceivce( address | 0x40 );
    
    /* д */
    while( i-- )
    {
        Ebyte_Port_SpiTransmitAndReceivce( *data++ );
    }
    
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(1);
}

/*!
 * @brief д
 * 
 * @param address Ĵַ
 */
static void E07x_SendCommand( uint8e_t command )
{
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(0);
    
    /* д */
    Ebyte_Port_SpiTransmitAndReceivce( command );
    
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(1);
}

/*!
 * @brief FIFOд
 * 
 * @param data ָ
 * @param size д볤
 */
static void E07x_SetFIFO( uint8e_t *data, uint8e_t size)
{
    uint8e_t i = size;
    
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(0);
    
    /* FIFOַ:0x3F ַbit6λдָ |0x40 */
    Ebyte_Port_SpiTransmitAndReceivce( 0x3F | 0x40 );
    
    /* д */
    while( i-- )
    {
        Ebyte_Port_SpiTransmitAndReceivce( *data++ );
    }
    
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(1);
}

/*!
 * @brief FIFOȡ
 * 
 * @param data ָ򻺴
 * @param size ȡ
 */
static void E07x_GetFIFO( uint8e_t *data, uint8e_t size)
{
    uint8e_t i = size;
    
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(0);
    
    /* FIFOַ:0x3F ַbit6λָ |0x40 */
    Ebyte_Port_SpiTransmitAndReceivce( 0x3F | 0xC0 );
    
    /* д */
    while( i-- )
    {
        *data++ = Ebyte_Port_SpiTransmitAndReceivce( 0xFF );
    }
    
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(1);
}

/*!
 * @brief FIFO
 * @param mode ģʽ
 *           @arg FIFO_WRITE ָFIFO
 *           @arg FIFO_READ  ָFIFO
 */
static void E07x_ClearFIFO( uint8e_t mode )
{
  
    if( mode == FIFO_WRITE )
    {    
        /* ָ:FIFO(0x3B) */
        E07x_SendCommand(0x3B);
    }
    else
    {
        /* ָ:FIFO(0x3A) */
        E07x_SendCommand(0x3A);
    } 
    
}

/*!
 * @brief λ
 */
void E07x_Reset(void)
{
    /* SPI CS */
    Ebyte_Port_SpiCsIoControl(1);
    Ebyte_Port_SpiCsIoControl(0);
    Ebyte_Port_DelayMs(1);
    Ebyte_Port_SpiCsIoControl(1);
    Ebyte_Port_DelayMs(1);
    
    /* :оƬλ(0x30) */
    E07x_SendCommand( 0x30 );
    
    Ebyte_Port_DelayMs(20);
}

/*!
 * @brief лģʽ
 */
static void E07x_SetStby(void)
{
    /* ָ:0x36 ˳TX/RX ģʽ ״̬ */
    E07x_SendCommand(0x36);
    
    /* ״̬¼ */
    E07x_Status = GO_STBY;
}

/*!
 * @brief лģʽ
 * 
 * @note ֲᣬּĴĲᶪʧ
 */
static void E07x_SetSleep(void)
{
    /* ָ:0x39 ģʽ */
    E07x_SendCommand(0x39);
    
    /* ״̬¼ */
    E07x_Status = GO_SLEEP;
}

static void E07x_SetGPIO( uint8e_t gpio , uint8e_t config )
{
  
  /* ָIOļ жʱΪߵƽ жʱΪ͵ƽ */
  uint8e_t mask = 0x40;
  
  /* ϲж */ 
  mask |= (config & 0x3F);
  
  switch ( gpio )
  {
    /* IOCFG0 Ĵַ:0x02 */
    case 0:  E07x_SetRegister(0x02,mask) ;break;
    /* IOCFG1 Ĵַ:0x01 */
    case 1:  E07x_SetRegister(0x01,mask) ;break;
    /* IOCFG2 Ĵַ:0x00 */
    case 2:  E07x_SetRegister(0x00,mask) ;break;
    default: break;
  }
}

/*!
 * @brief жģǷ
 *
 * @return 0: 1:ģ쳣
 */
static uint8e_t E07x_IsExist(void)
{
    uint8e_t result = 0;
    uint8e_t reg_value;
    
    /* ¶ȴò λӦΪ0x7F Ĵַ:0x2A  */
    reg_value = E07x_GetRegister(0x2A);
    
    /* ĬֵΪ0x7F */
    if( reg_value != 0x7F ) 
    {
       /* ȷ Ӳ */
       result = 1;
    }
    
    return result;
}

/*!
 * @brief Ʒʽ
 *
 * @param mod ĵƷʽ 
 *        @arg 0:2-FSK
 *        @arg 1:GFSK
 *        @arg 3:ASK/OOK
 *        @arg 4:4-FSK
 *        @arg 7:MSK
 */
static void E07x_SetModulation(uint8e_t mod)
{
    /* תλ */
    uint8e_t tmp = (mod<<4) & 0x70;
  
    /* MDMCFG2 Ĵַ 0x12  bit[6:4]*/
    E07x_InitSetting.mdmcfg2  &= 0x8F;
    
    /* bit[6:4] */
    E07x_InitSetting.mdmcfg2 |= tmp;
}

/*!
 * @brief زƵʻ׼ֵ
 *
 * @param frequency زƵ Hz
 * @return 0: 1:
 * @note Ƶ֮( Hz ):
 *        300000000 - 348000000
 *        387000000 - 464000000
 *        779000000 - 928000000
 *       ע:ܹͬСֿܲһ£пܲƫ
 *
 *       ƵǲƵOKʵٷƵʵģͨš
 */
static uint8e_t E07x_SetFrequency( uint32e_t frequency )
{ 
   uint32e_t freq ;
   uint8e_t  freq2,freq1,freq0;
   
   /*  */
   if(  frequency > 928000000 ) return 1;
   if( (frequency < 779000000) && ( frequency>464000000) )return 1;
   if( (frequency < 387000000) && ( frequency>348000000) )return 1;
   if( (frequency < 300000000)) return 1;
   
   /* Ƶʿ(FREQ)=زƵ *2^16)/ Ƶ   */
   freq = (uint32e_t)(((double)frequency * 65535 )/ ( (double)EBYTE_E07X_XOSC ));
   freq2 = ( uint8e_t )( ( freq >> 16 ) & 0xFF );
   freq1 = ( uint8e_t )( ( freq >> 8 ) & 0xFF );
   freq0 = ( uint8e_t )(  freq  & 0xFF );
   
   /* ʼеƵʲ ȴͳһд  */
   E07x_InitSetting.freq2 = freq2;
   E07x_InitSetting.freq1 = freq1;
   E07x_InitSetting.freq0 = freq0;
   
   return 0;
}

/*!
 * @brief ʻ׼ֵ
 *
 * @param datarate 
 * @note  ֮:
 *        300 (0.3K) - 250000000 (250K)
 */
static void E07x_SetDataRate( uint32e_t datarate )
{
    uint8e_t  date_m ;
    uint16e_t date_e ;
    uint32e_t date_e_sum = 1,temp;
    
    /* м乫ʽ (256+DRATE_M)*2^DRATE_E=datarate*2^28/Ƶ  */
    uint32e_t calculation = (uint32e_t)( datarate * EBYTE_E07X_CAL_BASE);
    
    /* αƽƥ DRATE_E */
    for( date_e=0 ; date_e<=0xFF;date_e++ )
    {
       if(date_e==0)
       {
          date_e_sum =1;//2^0
       }
       else
       {
          date_e_sum *= 2;//2^DRATE_E 
       }
       
       temp = calculation/date_e_sum; //  [256 ,256+DRATE_M]֮
       if(  temp>=256 && temp<=511 )
       {
          date_m = temp - 256;//õDRATE_M
          break;
       }
    }
    
    /* ʼеʲ ȴͳһд  */
    if ( date_e<256 )
    {
       E07x_InitSetting.mdmcfg4 &= 0xF0;
       E07x_InitSetting.mdmcfg4 |= date_e;
       E07x_InitSetting.mdmcfg3 = date_m;    
    }
    
}

/*!
 * @brief Ƶ׼ֵ
 *
 * @param bandwidth ĽƵ (Hz)
 * @note ʵ͹ٷһ Ĭ58.035714
 *       ע⣺оƬʵֻΪ̶Ƶֵᱻתӽ¹̶Ƶ
 *        58.035714 KHz 
 *        67.708333 KHz 
 *        81.250000 KHz
 *       101.562500 KHz 
 *       116.071429 KHz
 *       135.416667 KHz
 *       162.500000 KHz
 *       203.125000 KHz
 *       232.142857 KHz
 *       270.833333 KHz
 *       325.000000 KHz
 *       406.250000 KHz
 *       464.285714 KHz
 *       541.666667 KHz
 *       650.000000 KHz
 *       812.500000 KHz
 */
static void E07x_SetChannelBandwidth( uint32e_t bandwidth )
{
    uint8e_t  chanbw_m ,chanbw_e,mask;

    uint32e_t chanbw_e_sum = 1,temp;
    
    /* м乫ʽ (4+CHANBW_M)*2^CHANBW_E=Ƶ/(8*bandwidth)  */
    uint32e_t calculation = (uint32e_t)( EBYTE_E07X_XOSC/(8 * bandwidth));    
    
    /* αƽƥ CHANBW_E */
    for( chanbw_e=0 ; chanbw_e<=3;chanbw_e++ )
    {
       if(chanbw_e==0)
       {
          chanbw_e_sum = 1;//2^0
       }
       else
       {
          chanbw_e_sum *= 2;//2^CHANBW_E
       }
        
       temp = calculation/chanbw_e_sum; 
       if(  temp>=4 && temp<=7 )
       {
          chanbw_m = temp - 4;//õCHANBW_M
          break;
       }
    }    
    
    /* ʼеƵ ȴͳһд  */
    if( chanbw_e<=3 )
    {
        mask = ((uint8e_t)((chanbw_e<<6)|(chanbw_m<<4))) & 0xF0;
        E07x_InitSetting.mdmcfg4 &= 0x0F;
        E07x_InitSetting.mdmcfg4 |= mask;
    }
    
}

/*!
 * @brief Ƶƫ׼ֵ
 * 
 * @param frequency_dev λ:Hz 
 * 
 * @note ʵ͹ٷһ
 */
static void E07x_SetFrequencyDeviation( uint32e_t frequency_dev )
{
    uint8e_t  deviation_m,deviation_e,mask;
    uint32e_t deviation_e_sum = 1,temp;
    
    /* м乫ʽ (8+DEVIATION_M)*2^DEVIATION_E=frequency_dev * 2^17 / Ƶ  */
    uint32e_t calculation = (uint32e_t)( frequency_dev * EBYTE_E07X_POW_2_17 / EBYTE_E07X_XOSC );   
    
    /* αƽƥ DEVIATION_E */
    for( deviation_e=0 ; deviation_e<=7;deviation_e++ )
    {
       if(deviation_e==0)
       {
          deviation_e_sum = 1;//2^0
       }
       else
       {
          deviation_e_sum *= 2;//2^DEVIATION_E
       }
        
       temp = calculation/deviation_e_sum; //õ 8+DEVIATION_M
       
       if(  temp>=8 && temp<=15 )
       {
          deviation_m = temp - 8;//õCHANBW_M
          break;
       }
    }      
    
    /* ʼеƵƫ ȴͳһд  */
    if( deviation_e<=7 )
    {
        mask = ((uint8e_t)((deviation_e<<4)|deviation_m)) & 0x77;
        E07x_InitSetting.deviatn &= 0x88;
        E07x_InitSetting.deviatn |= mask;
    }    
}

/*!
 * @brief 
 * 
 * @param power λ dBm ע:ֵ  
 * 
 * @return 0 1:ȷ
 * @note PA RampingҲʧȥһԶȺԣɢźӣ 
 *       ʵԵģõPAֵTI SmartRF StudioΪο
 */
static uint8e_t E07x_SetOutputPower( int8e_t power )
{
    uint8e_t result = 0;
#if defined(EBYTE_E07_400M10S)
    switch ( power ) 
    {
        case 10:  E07x_PaTabel[0] = 0xC0;break;
        case 7:   E07x_PaTabel[0] = 0xC8;break;
        case 5:   E07x_PaTabel[0] = 0x84;break;
        case 0:   E07x_PaTabel[0] = 0x60;break;
        case -10: E07x_PaTabel[0] = 0x34;break;
        case -15: E07x_PaTabel[0] = 0x1D;break;
        case -20: E07x_PaTabel[0] = 0x0E;break;
        case -30: E07x_PaTabel[0] = 0x12;break;        
        default: result=1; break;
    }
#elif defined(EBYTE_E07_900M10S)  
    
    if( E07_FREQUENCY_START > 900000000 ) //900MHz  928MHz
    {
        switch ( power ) 
        {
            case 11:  E07x_PaTabel[0] = 0xC0;break; //Ƶ ʿ11dBm    
            case 10:  E07x_PaTabel[0] = 0xC3;break;
            case 7:   E07x_PaTabel[0] = 0xCC;break;
            case 5:   E07x_PaTabel[0] = 0x84;break;
            case 0:   E07x_PaTabel[0] = 0x8E;break;
            case -10: E07x_PaTabel[0] = 0x27;break;
            case -15: E07x_PaTabel[0] = 0x1E;break;
            case -20: E07x_PaTabel[0] = 0x0E;break;
            case -30: E07x_PaTabel[0] = 0x03;break;        
            default: result=1; break;
        }    
    }
    else //779MHz  900MHz
    {
        switch ( power ) 
        {
            case 12:  E07x_PaTabel[0] = 0xC0;break; //Ƶ ʿ12dBm    
            case 10:  E07x_PaTabel[0] = 0xC5;break;
            case 7:   E07x_PaTabel[0] = 0xCD;break;
            case 5:   E07x_PaTabel[0] = 0x86;break;
            case 0:   E07x_PaTabel[0] = 0x50;break;
            case -10: E07x_PaTabel[0] = 0x26;break;
            case -15: E07x_PaTabel[0] = 0x1D;break;
            case -20: E07x_PaTabel[0] = 0x17;break;
            case -30: E07x_PaTabel[0] = 0x03;break;        
            default: result=1; break;
        }            
    }

#endif    

    
    return result;
}

/*!
 * @brief ģ
 * 
 * @param frequency      زƵ
 * @param data_rate      ()
 * @param frequency_dev  Ƶƫ
 * @param bandwidth      Ƶ
 * @param output_power   
 * @param preamble_size  ǰ볤 
 * @param sync_word      ͬ
 * @param crc            CRC
 * @param device_address 豸ַ
 *
 * @return 0: 1:
 * @note (ѡ)ĬϹر˵ַ
 */
static uint8e_t E07x_Config( uint32e_t frequency , uint32e_t data_rate , uint32e_t frequency_dev ,uint32e_t bandwidth, 
                              int8e_t output_power , uint16e_t preamble_size, uint16e_t sync_word , uint8e_t crc )
{
    uint8e_t result ;  
    uint8e_t reg_value;
    
    /* :زƵ 
     * Ĵʼַ 0x0D */
    result = E07x_SetFrequency( frequency );
    if( result != 0 ) return 1;
    
    /* : 
     * MDMCFG4 MDMCFG3Ĵַ:0x10 0x11 */
    E07x_SetDataRate( data_rate );
    
    /* :Ƶƫ 
     * DEVIATNĴַ:0x15 */
    E07x_SetFrequencyDeviation( frequency_dev );    
    
    /* :Ƶ 
     * MDMCFG4Ĵַ:0x10 */
    E07x_SetChannelBandwidth( bandwidth );
    
    /* : 
     * Ĵַ:0x3E */
    E07x_SetOutputPower(output_power);
    
    /* ģʽ
     * MDMCFG2 Ĵַ:0x12 */
    E07x_SetModulation(1);//GFSK
    
    /* ǰ볤 
     * MDMCFG1 Ĵַ:0x13 */  
    if( preamble_size >7 ) return 1;//
    E07x_InitSetting.mdmcfg1 &= 0x8F;//
    E07x_InitSetting.mdmcfg1 |= (preamble_size<<4);//λ
    
    /* ͬ 
     * SYNC1 SYNC0 Ĵַ:0x04 0x05 */
    E07x_InitSetting.sync1 =  (uint8e_t)((sync_word>>8)&0xFF);//Byte
    E07x_InitSetting.sync0 =  (uint8e_t)(sync_word & 0xFF);   //Byte
    
    /* CRC 
     * PKTCTRL0Ĵַ:0x08 bit2 */
    if( crc > 1 ) return 1;// ֻΪ01
    if( crc )
    {
        E07x_InitSetting.pktctrl0 |= 0x04; // 
    }else
    {
        E07x_InitSetting.pktctrl0 &= 0xFB;
    }
    
    /* ݰ ĬΪɱ䳤ģʽҳȷݵһֽ
     * PKTCTRL0Ĵַ:0x08 bit[1:0] */
    E07x_InitSetting.pktctrl0 &= 0xFC;//
    E07x_InitSetting.pktctrl0 |= 0x01;//0x01ģʽ (ɱ䳤)
    
    /* ע:ϣȫʹùٷSmartRF Studioòôֱע͵Ϸ޸ĺ */
    /* ģд E07x_InitSetting  */
    E07x_SetRegister( 0x02, E07x_InitSetting.iocfg0 );
    E07x_SetRegister( 0x03, E07x_InitSetting.fifothr);
    E07x_SetRegister( 0x04, E07x_InitSetting.sync1);
    E07x_SetRegister( 0x05, E07x_InitSetting.sync0);    
    E07x_SetRegister( 0x08, E07x_InitSetting.pktctrl0);
    E07x_SetRegister( 0x0B, E07x_InitSetting.fsctrl1);
    E07x_SetRegister( 0x0D, E07x_InitSetting.freq2);
    E07x_SetRegister( 0x0E, E07x_InitSetting.freq1);
    E07x_SetRegister( 0x0F, E07x_InitSetting.freq0);
    E07x_SetRegister( 0x10, E07x_InitSetting.mdmcfg4);
    E07x_SetRegister( 0x11, E07x_InitSetting.mdmcfg3);
    E07x_SetRegister( 0x12, E07x_InitSetting.mdmcfg2);
    E07x_SetRegister( 0x13, E07x_InitSetting.mdmcfg1);
    E07x_SetRegister( 0x15, E07x_InitSetting.deviatn);
    E07x_SetRegister( 0x18, E07x_InitSetting.mcsm0);
    E07x_SetRegister( 0x19, E07x_InitSetting.foccfg);
    E07x_SetRegister( 0x1B, E07x_InitSetting.agcctrl2);
    E07x_SetRegister( 0x1C, E07x_InitSetting.agcctrl1);
    E07x_SetRegister( 0x20, E07x_InitSetting.worctrl);
    E07x_SetRegister( 0x23, E07x_InitSetting.fscal3);
    E07x_SetRegister( 0x24, E07x_InitSetting.fscal2);
    E07x_SetRegister( 0x25, E07x_InitSetting.fscal1);
    E07x_SetRegister( 0x26, E07x_InitSetting.fscal0);
    E07x_SetRegister( 0x2C, E07x_InitSetting.test2);
    E07x_SetRegister( 0x2D, E07x_InitSetting.test1);
//    E07x_SetRegister( 0x2E, E07x_InitSetting.test0); //: TEST0ԶƵƫ  
                                                       //: ƵΪС411MHzʱƵ411MHzƵΪ411MHzʱƵл
                                                       //: ȫùٷSmartRF StudioʱTEST0ע͡ʹɲʱע
    
    /* 䣺ͨ(Ƶ)  Ĵַ: 0x0A */
    E07x_SetRegister( 0x0A ,0 );
    
    /* 䣺رյַ  Ĵַ: 0x07 */
    reg_value = E07x_GetRegister(0x07);
    reg_value &= 0xFC;//
    E07x_SetRegister(0x07,reg_value);
    
    /* :  Ĵַ: 0x3E */
    E07x_SetRegisters( 0x3E ,E07x_PaTabel,8 );
    
    return 0;
}

/*!
 * @brief ģʼ
 * 
 * @return 0: 1:ʼʧ
 */
uint8e_t E07x_Init( void )
{
    uint8e_t result = 0;
    
    /* λ */
    E07x_Reset();
    
    /* жģǷ */
    result = E07x_IsExist();
    if( result != 0 ) return 1;
    
    /*  */
    result = E07x_Config(  E07_FREQUENCY_START,
                           E07_DATA_RATE, 
                           E07_FREQUENCY_DEVIATION,
                           E07_BANDWIDTH,
                           E07_OUTPUT_POWER,
                           E07_PREAMBLE_SIZE,
                           E07_SYNC_WORD,
                           E07_IS_CRC
                           );
    if( result != 0 ) return 1;
    
    /* ĬϽģʽ */
    E07x_GoReceive();
    return 0;
}

/*!
 * @brief  
 */
void E07x_TaskForIRQ(void)
{

}

/*!
 * @brief ѯ ݽ
 *  
 * @param data ָ
 * @param size ݳ
 * @return 0
 */
uint8e_t E07x_GoTransmit( uint8e_t *data, uint8e_t size )
{
    uint8e_t irqStatus = 0;
    
    /* ģʽл:  */
    E07x_SetStby();
    
    /* ݶ */
    E07x_ClearFIFO( FIFO_WRITE );
   
    /* Ϊɱ䳤ģ һֽд볤 */
    E07x_SetFIFO( &size , 1 );
    
    /* дݰ */
    E07x_SetFIFO( data , size);
    
    /*  GPIO0 ӳ䵽ж
     * ͽͬʱ ж ע:FIFOеȫͺ Զж */
    E07x_SetGPIO(0 ,IRQ_TX_SYNC_SEND );

    /* ָ:0x35 ģģʽ */   
    E07x_SendCommand( 0x35 );      
    
    /* ״̬¼ */
    E07x_Status = GO_WAIT_TRANSMIT;    
    
    /* ͷʽ ڴ˵ȴ */  
    do
    {
      Ebyte_Port_DelayMs(1);
      
      /* ȡ״̬Ĵ MARCSTATE ַ:0x35 */
      irqStatus = E07x_GetRegister(0x35);

    }
    while( (irqStatus != 0x01) );//ɺԶ˻ص״̬ Ϊ0x01  
                                  //˼ĴMCSM1 ַ:0x17 TXOFF_MODE ôɺ״̬һǿ Ҫд 
    
    /* ״̬¼ */
    E07x_Status = GO_STBY;  
    
    /* صû */
    Ebyte_Port_TransmitCallback( 0x0001 );
    
    return 0;
}

/*!
 * @brief ʼ
 *  
 * @return 0
 */
uint8e_t E07x_GoReceive( void )
{
  
    /* ģʽл:  */
    E07x_SetStby();
    
    /* ݶ */
    E07x_ClearFIFO( FIFO_READ );  
  
    /*  GPIO0 ӳ䵽ж */
//    E07x_SetGPIO(0 ,IRQ_RX_CRC_OK );
     E07x_SetGPIO(0 ,IRQ_RX_SYNC_RECV );
  
    /* ָ:0x34 ģģʽ */   
    E07x_SendCommand( 0x34 );  
    
    /* ״̬¼ */
    E07x_Status = GO_WAIT_RECEIVE;
    return 0;
}

/*!
 * @brief ģģʽ(͹)
 * 
 * @return 0;
 */
uint8e_t E07x_GoSleep(void)
{
    E07x_SetSleep();
    
    return 0;
}

/*!
 * @brief ȡģ汾
 * 
 * @return 8λı 
 * @note 0x10 V1.0
 */
uint8e_t E07x_GetDriverVersion(void)
{
  return  EBYTE_E07_TYPE_PROGRAM;
}

/*!
 * @brief ȡģ
 * 
 * @return ַָָ 
 * @note ׼ַ ĩβн '\0'
 */
uint8e_t* E07x_GetName(void)
{
    return E07x_NameString;
}

/*!
 * @brief ȡģ״̬
 * 
 * @return ״̬
 *        δʼ     GO_INIT          =0x00        
 *        л״̬ GO_BUSY          =0x01   
 *        /    GO_STBY          =0x02   
 *        ׼ջ GO_RECEIVE       =0x03   
 *        ڼ GO_WAIT_RECEIVE  =0x04          
 *        ׼ͻ GO_TRANSMIT      =0x05   
 *        ȴ GO_WAIT_TRANSMIT =0x06   
 *                 GO_SLEEP         =0x07   
 *        ڲ     GO_ERROR         =0x08 
 */
uint8e_t E07x_GetStatus(void)
{
  return (uint8e_t)E07x_Status;;
}

/*!
 * @brief ѯ ݽ
 *  
 * @return 0
 * @note ҪԵ
 */
uint8e_t E07x_TaskForPoll(void)
{
    
    if (0 == Ebyte_Port_Gdo0IoRead())             // ģǷж 
    {
        while (Ebyte_Port_Gdo0IoRead() == 0);

        // ȡյݳ 
        uint8e_t recvSize = 0;
        E07x_GetFIFO(&recvSize , 1);

        // ճȲΪ0
        if( recvSize != 0 )
        {
            E07x_GetFIFO( E07x_RxBuffer, recvSize );
            //صû
            Ebyte_Port_ReceiveCallback( 0x0002 , E07x_RxBuffer , recvSize );            
        }
        else
        {
            //To-Do д ֻǰ?͵жϣ
            E07x_GoReceive();//ʾ 
        }
    }

    return 0;
}