/**
  **********************************************************************************
  * @file      ebyte_E29x.c
  * @brief     EBYTE E29 ϵ
  * @details   μ https://www.ebyte.com/
  * @author    Weng
  * @date      2022-11-29
  * @version   1.0.0
  **********************************************************************************
  * @copyright BSD License
  *            ɶڰصӿƼ޹˾
  *   ______   ____   __     __  _______   ______
  *  |  ____| |  _ \  \ \   / / |__   __| |  ____|
  *  | |__    | |_) |  \ \_/ /     | |    | |__
  *  |  __|   |  _ <    \   /      | |    |  __|
  *  | |____  | |_) |    | |       | |    | |____
  *  |______| |____/     |_|       |_|    |______|
  *
  **********************************************************************************
  */

#include "ebyte_e29x.h"

// ʶ룺ģ
#if defined(EBYTE_E29_400M22S)
static uint8e_t E29x_NameString[] = "E29-400M22S";
#elif defined(EBYTE_E29_400M22S)
static uint8e_t E29x_NameString[] = "E29-400M22S";
#endif

// ʶ룺汾
#define EBYTE_E29_PROGRAM_TYPE 0x10

// ģڲշ·ģʽ
#define ANTENNA_RXEN  0
#define ANTENNA_TXEN  1

// жϼĴʶ ֵ֧
#define REG_IRQ_RX_PLHD_DONE                0x10
#define REG_IRQ_RX_DONE                     0x08
#define REG_IRQ_CRC_ERR                     0x04
#define REG_IRQ_RX_TIMEOUT                  0x02
#define REG_IRQ_TX_DONE                     0x01

#define MODULE_GPIO_TX                      3
#define MODULE_GPIO_RX                      1
#define MODULE_GPIO_TCXO                    5
#define MODULE_GPIO_CAD_IRQ                 11

#define RF_MODE_DEEP_SLEEP                  0
#define RF_MODE_SLEEP                       1
#define RF_MODE_STB1                        2
#define RF_MODE_STB2                        3
#define RF_MODE_STB3                        4
#define RF_MODE_TX                          5
#define RF_MODE_RX                          6


typedef enum
{
    GO_INIT          = 0x00,
    GO_BUSY          = 0x01,
    GO_STBY          = 0x02,
    GO_RECEIVE       = 0x03,
    GO_WAIT_RECEIVE  = 0x04,
    GO_TRANSMIT      = 0x05,
    GO_WAIT_TRANSMIT = 0x06,
    GO_SLEEP         = 0x07,
    GO_ERROR         = 0x08
} E29x_Status_t;

enum PAGE_SEL
{
    PAGE0_SEL,
    PAGE1_SEL,
    PAGE2_SEL,
    PAGE3_SEL
};

// ״̬ʶ
static E29x_Status_t E29x_Status = GO_INIT;

// ݻ
static uint8e_t E29x_ReceiveBuffer[255] ;

// AGCĴֵ
uint8e_t reg_agc_value[98] = {0x90, 0xff, 0x64, 0x24, 0x00, 0x03, 0x24, 0x24, 0x00, 0x03, 0x24, 0x24, 0x00, 0x03, 0x24, 0x24, \
                              0x00, 0x03, 0x24, 0x24, 0x00, 0x03, 0x24, 0x24, 0x00, 0x03, 0x24, 0x24, 0x00, 0x03, 0x24, 0x24, \
                              0x00, 0x03, 0x24, 0x24, 0x00, 0x03, 0x24, 0x24, 0x00, 0x03, 0x24, 0x24, 0x00, 0x03, 0x24, 0x2A, \
                              0x20, 0x03, 0x2A, 0x30, 0x40, 0x03, 0x30, 0x36, 0x60, 0x03, 0x36, 0x3B, 0x80, 0x03, 0x3B, 0x41, \
                              0x84, 0x03, 0x41, 0x46, 0x88, 0x03, 0x46, 0x4B, 0x8C, 0x03, 0x4B, 0x50, 0x90, 0x03, 0x50, 0x56, \
                              0x91, 0x03, 0x56, 0x06, 0xFF, 0x40, 0x42, 0x0F, 0x00, 0x00, 0x01, 0xF4, 0x2F, 0xF3, 0x0F, 0x00, 0x00, 0x00
                             };

/*!
 * @brief дĴ
 *
 * @param address Ĵʼַ
 * @param data    дָ
 * @param size    Ĵ
 */
static void E29x_SetRegisters( uint8e_t address, uint8e_t* data, uint8e_t size )
{
    uint8e_t i;
    uint8e_t addr_w = ( 0x01 | ( address << 1 ) );
    /* CSƬѡ ͵ƽѡ */
    Ebyte_Port_SpiCsIoControl( 0 );
    /* Ĵʼַ */
    Ebyte_Port_SpiTransmitAndReceivce( addr_w );
    /* д */
    for( i = 0; i < size; i++ )
    {
        Ebyte_Port_SpiTransmitAndReceivce( data[i] );
    }
    /* CSƬѡ ߵƽ */
    Ebyte_Port_SpiCsIoControl( 1 );
}

/*!
 * @brief ȡĴ
 *
 * @param address Ĵʼַ
 * @param data    ָȡݴŻ
 * @param size    Ĵ
 */
static void E29x_GetRegisters( uint8e_t address, uint8e_t* data, uint8e_t size )
{
    uint8e_t i;
    uint8e_t addr_r = ( 0x00 | ( address << 1 ) );
    /* CSƬѡ ͵ƽѡ */
    Ebyte_Port_SpiCsIoControl( 0 );
    /* Ĵʼַ */
    Ebyte_Port_SpiTransmitAndReceivce( addr_r );
    /* ȡ */
    for( i = 0; i < size; i++ )
    {
        data[i] = Ebyte_Port_SpiTransmitAndReceivce( 0 );
    }
    /* CSƬѡ ߵƽ */
    Ebyte_Port_SpiCsIoControl( 1 );
}

/*!
 * @brief д뵥Ĵ
 *
 * @param address Ĵʼַ
 * @param data    д
 */
static void E29x_SetRegister( uint8e_t address, uint8e_t data )
{
    E29x_SetRegisters( address, &data, 1 );
}

/*!
 * @brief ȡĴ
 *
 * @param address Ĵʼַ
 * @param data    д
 */
static uint8e_t E29x_GetRegister( uint8e_t address )
{
    uint8e_t data;
    E29x_GetRegisters( address, &data, 1 );
    return data;
}

/*!
 * @brief лĴҳ
 *
 * @param page  лĴҳ
 * @return      ɱ־λ:OKɹ  FAILʧ
 */
static uint8e_t E29x_Switch_Page( enum PAGE_SEL page )
{
    uint8e_t page_sel = 0x00;
    uint8e_t tmpreg   = 0x00;
    /* ȡĴ0x00Ĳ */
    tmpreg = E29x_GetRegister( 0x00 );
    /* ޸ļĴ0x00Ĳ */
    page_sel  = ( tmpreg & 0xfc ) | page;
    E29x_SetRegister( 0x00, page_sel );
    /* ֤Ƿ޸ĳɹ */
    if( ( E29x_GetRegister( 0x00 ) & 0x03 ) == page )
    {
        return OK;
    }
    else
    {
        return FAIL;
    }
}

/*!
 * @brief лĴҳдضַļĴ
 *
 * @param page    Ĵҳ
 * @param address Ĵʼַ
 * @param data    д
 * @return        ɱ־λ:OKɹ  FAILʧ
 */
static uint8e_t E29x_Set_Page_Register( enum PAGE_SEL page, uint8e_t addr, uint8e_t value )
{
    uint8e_t tmpreg;
    /* ҳǷлɹ */
    if( E29x_Switch_Page( page ) != OK )
    {
        return FAIL;
    }
    /* дȡǷȷ */
    E29x_SetRegister( addr, value );
    tmpreg = E29x_GetRegister( addr );
    if( tmpreg == value )
    {
        return OK;
    }
    else
    {
        return FAIL;
    }
}

/*!
 * @brief лĴҳضַļĴ
 *
 * @param page    Ĵҳ
 * @param address Ĵʼַ
 * @return        Ĵֵ
 */
static uint8e_t E29x_Get_Page_Register( enum PAGE_SEL page, uint8e_t addr )
{
    /* ҳǷлɹ */
    if( E29x_Switch_Page( page ) != OK )
    {
        return FAIL;
    }
    /* ضȡĲ */
    return E29x_GetRegister( addr );
}

/*!
 * @brief лĴҳдĴ
 *
 * @param page    Ĵҳ
 * @param address Ĵʼַ
 * @return        ɱ־λ:OKɹ  FAILʧ
 */
static uint8e_t E29x_Set_Page_Register_Continue( enum PAGE_SEL page, uint8e_t addr, uint8e_t* buffer, uint8e_t len )
{
    /* ҳǷлɹ */
    if( E29x_Switch_Page( page ) != OK )
    {
        return FAIL;
    }
    /* дĴȡĴУ */
    uint8e_t i, temreg[100];
    E29x_SetRegisters( addr, buffer, len );
    E29x_GetRegisters( addr, temreg, len );
    for( i = 0; i < len; i++ )
    {
        if( buffer[i] != temreg[i] )
        {
            return FAIL;
        }
    }
    return OK;
}

/*!
 * @brief 趨RFΪģʽ
 *
 * @param gpio_pin    ű
 * @return            ɱ־λ:OKɹ  FAILʧ
 */
static uint8e_t E29x_Set_GPIO_Output( uint8e_t gpio_pin )
{
    uint8e_t tmpreg = 0;
    if( gpio_pin < 8 )
    {
        /* GPIO0 ~ GPIO7 ʹ */
        tmpreg = E29x_Get_Page_Register( PAGE0_SEL, 0x65 );
        tmpreg |= ( 1 << gpio_pin );
        /* Ƿʹܳɹ */
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x65, tmpreg ) != OK )
        {
            return FAIL;
        }
        else
        {
            return OK;
        }
    }
    else
    {
        /* GPIO8 ~ GPIO11 ʹ */
        tmpreg = E29x_Get_Page_Register( PAGE0_SEL, 0x66 );
        tmpreg |= ( 1 << ( gpio_pin - 8 ) );
        /* Ƿʹܳɹ */
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x66, tmpreg ) != OK )
        {
            return FAIL;
        }
        else
        {
            return OK;
        }
    }
}

/*!
 * @brief 趨RFƽ
 *
 * @param gpio_pin    ű
 * @param state       ״̬
 * @return            ɱ׼λ:OKɹ  FAILʧ
 */
static uint8e_t E29x_Set_GPIO_Status( uint8e_t gpio_pin, uint8e_t state )
{
    uint8e_t tmpreg = 0;
    if( gpio_pin < 8 )
    {
        /* 趨 GPIO0 ~ GPIO7 ƽ */
        tmpreg = E29x_Get_Page_Register( PAGE0_SEL, 0x67 );
        if( state == 0 )
        {
            tmpreg &= ~( 1 << gpio_pin );
        }
        else
        {
            tmpreg |= ( 1 << gpio_pin );
        }
        /* Ƿƽóɹ */
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x67, tmpreg ) != OK )
        {
            return FAIL;
        }
        else
        {
            return OK;
        }
    }
    else
    {
        /* 趨 GPIO8 ~ GPIO11 ƽ */
        tmpreg = E29x_Get_Page_Register( PAGE0_SEL, 0x68 );
        if( state == 0 )
        {
            tmpreg &= ~( 1 << ( gpio_pin - 8 ) );
        }
        else
        {
            tmpreg |= ( 1 << ( gpio_pin - 8 ) );
        }
        /* Ƿƽóɹ */
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x68, tmpreg ) != OK )
        {
            return FAIL;
        }
        else
        {
            return OK;
        }
    }
}

/*!
 * @brief лģڲշ·
 *
 * @param mode лָģʽ
 *      @arg ANTENNA_RXEN: ģʽ
 *      @arg ANTENNA_TXEN: ģʽ
 *
 * @note շ·лȷᵼ·书½ ޷շ
 */
static void E29x_SetAntenna( uint8e_t mode )
{
    if( mode == ANTENNA_TXEN )
    {
        /* ʹ */
        //E29x_Set_GPIO_Output(MODULE_GPIO_RX);
        //E29x_Set_GPIO_Output(MODULE_GPIO_TX);
        /* лڲշ· */
        E29x_Set_GPIO_Status( MODULE_GPIO_RX, 1 );
        E29x_Set_GPIO_Status( MODULE_GPIO_TX, 0 );
    }
    else
    {
        /* ʹ */
        //E29x_Set_GPIO_Output(MODULE_GPIO_RX);
        //E29x_Set_GPIO_Output(MODULE_GPIO_TX);
        /* лڲշ· */
        E29x_Set_GPIO_Status( MODULE_GPIO_TX, 1 );
        E29x_Set_GPIO_Status( MODULE_GPIO_RX, 0 );
    }
}

/*!
 * @brief дģڲݶ
 *
 * @param data ָݰ
 * @param size дݳ
 *
 * @note ģлΪģʽ󣬻ԶӶȡݽߴ
 */
static void E29x_SetFIFO( uint8e_t* data, uint8e_t size )
{
    E29x_SetRegisters( 0x01, data, size );
}

/*!
 * @brief ȡģڲݶ
 *
 * @param data ָûݻ
 * @param size ݳ
 *
 * @note עʱ򣬽ģʽʱҪöȡַλȡʵʽճȣɺٶȡFIFO
 */
static void E29x_GetFIFO( uint8e_t* data, uint8e_t size )
{
    E29x_GetRegisters( 0x01, data, size );
}

/*!
 * @brief ģ鸴λ
 *
 * @note ڲĴỹԭΪĬֵ
 */
void E29x_Reset( void )
{
    uint8e_t tmpreg = 0;
    /* ȡĴֵ */
    tmpreg = E29x_GetRegister( 0x00 );
    /* λλ־ */
    tmpreg |= 0x80;
    E29x_SetRegister( 0x00, tmpreg );
    /* ٴζȡĴֵ */
    tmpreg = E29x_GetRegister( 0x00 );
    /*  */
    tmpreg &= 0x7F;
    E29x_SetRegister( 0x00, tmpreg );
}

/*!
 * @brief ȡжϱʶ
 *
 * @return һ㶼Ƿ/
 *       0x10:REG_IRQ_RX_PLHD_DONE
 *       0x08:REG_IRQ_RX_DONE
 *       0x04:REG_IRQ_CRC_ERR
 *       0x02:REG_IRQ_RX_TIMEOUT
 *       0x01:REG_IRQ_TX_DONE
 */
uint8e_t E29x_GetIRQ( void )
{
    /* ȡж״̬ Ĵַ:0x6C */
    return E29x_Get_Page_Register( PAGE0_SEL, 0x6C );
}

/*!
 * @brief жϱʶ
 */
void E29x_ClearIRQ( void )
{
    /* ѭ3ΣȷIRQ */
    uint8e_t  clr_cnt = 0;
    uint16e_t a = 0, b = 0;
    while( clr_cnt < 3 )
    {
        /* еIRQ־λ */
        E29x_Set_Page_Register( PAGE0_SEL, 0x6C, 0x1f );
        /* Ƿɹ */
        if( E29x_Get_Page_Register( PAGE0_SEL, 0x6C ) == 0 )
        {
            return ;
        }
        /* ʧܣʱһʱٴ */
        else
        {
            clr_cnt++;
            for( a = 0; a < 1200; a++ ) //10ms
                for( b = 0; b < 100; b++ );
            continue;
        }
    }
}

/*!
 * @brief ģʽ(ߣѺҪò)
 * @brief ûòģʽ
 */
void E29x_SetSleep( void )
{
    /* ״̬¼ */
    E29x_Status = GO_BUSY;
    /* лģʽΪSTB3 */
    E29x_SetRegister( 0x02, RF_MODE_STB3 ) ;
    Ebyte_Port_DelayUs( 10 );
    /* лģʽΪSTB2 */
    E29x_SetRegister( 0x02, RF_MODE_STB2 );
    Ebyte_Port_DelayUs( 10 );
    /* лģʽΪSTB1 */
    E29x_SetRegister( 0x02, RF_MODE_STB1 );
    Ebyte_Port_DelayUs( 10 );
    /* ѹ߼λ */
    E29x_SetRegister( 0x04, 0x16 );
    Ebyte_Port_DelayUs( 10 );
    /* л˯ģʽ */
    E29x_SetRegister( 0x02, RF_MODE_SLEEP );
    Ebyte_Port_DelayUs( 10 );
    /* л˯ģʽҪò*/
    E29x_SetRegister( 0x02, RF_MODE_DEEP_SLEEP );
    /* ״̬¼ */
    E29x_Status = GO_SLEEP;
}

/*!
 * @brief ˳ģʽ(ߣѺҪò)
 */
void E29x_SetSleep_Wakeup( void )
{
    uint8e_t rstreg = 0, porreg = 0;
    /* ״̬¼ */
    E29x_Status = GO_BUSY;
    /* ѹ߼λ */
    porreg = E29x_GetRegister( 0x04 );
    Ebyte_Port_DelayUs( 10 );
    porreg |= 0x10;
    E29x_SetRegister( 0x04, porreg );
    Ebyte_Port_DelayUs( 10 );
    porreg &= 0xEF;
    E29x_SetRegister( 0x04, porreg );
    /* λƵоƬ */
    rstreg = E29x_GetRegister( 0x00 );
    rstreg &= 0x7F;
    E29x_SetRegister( 0x00, rstreg );
    Ebyte_Port_DelayUs( 10 );
    rstreg |= 0x80;
    E29x_SetRegister( 0x00, rstreg );
    Ebyte_Port_DelayUs( 10 );
    rstreg &= 0x7F;
    E29x_SetRegister( 0x00, rstreg );
    Ebyte_Port_DelayUs( 10 );
    /* лģʽ˯ģʽ */
    E29x_SetRegister( 0x02, RF_MODE_DEEP_SLEEP );
    Ebyte_Port_DelayUs( 10 );
    /* лģʽ˯ģʽ */
    E29x_SetRegister( 0x02, RF_MODE_SLEEP );
    Ebyte_Port_DelayMs( 1 );
    /* õ͹LDOؼĴ */
    E29x_SetRegister( 0x03, 0x1b );
    /* LDOƽתѹ߼λ */
    E29x_SetRegister( 0x04, 0x76 );
    /* δ֪ */
    E29x_Set_Page_Register( PAGE3_SEL, 0x26, 0x40 );
    /* TXCOŹܣڲԴ񣬹ʴ˴û */
    //    E29x_Set_GPIO_Output(MODULE_GPIO_TCXO);
    //    E29x_Set_GPIO_Status(MODULE_GPIO_TCXO, 1);
    //    Ebyte_Port_DelayMs(1);
    /* лģʽSTB1 */
    E29x_SetRegister( 0x02, RF_MODE_STB1 );
    Ebyte_Port_DelayUs( 10 );
    /* лģʽSTB2 */
    E29x_SetRegister( 0x02, RF_MODE_STB2 );
    Ebyte_Port_DelayMs( 1 );
    /* лģʽSTB3 */
    E29x_SetRegister( 0x02, RF_MODE_STB3 );
    Ebyte_Port_DelayUs( 10 );
    /* ״̬¼ */
    E29x_Status = GO_STBY;
}

/*!
 * @brief ģʽ
 */
void E29x_SetStby( void )
{
    /* ״̬¼ */
    E29x_Status = GO_BUSY;
    /* лSTB3ģʽ */
    E29x_SetRegister( 0x02, RF_MODE_STB3 );
    Ebyte_Port_DelayUs( 10 );
    /* ״̬¼ */
    E29x_Status = GO_STBY;
}

/*!
 * @brief 뷢ģʽ
 */
void E29x_SetTransmit( void )
{
    uint8e_t temp_val_1;
    uint8e_t temp_val_2;
    uint8e_t tmp;
    /* л͵· */
    E29x_SetAntenna( ANTENNA_TXEN );
    /* TXVCO */
    temp_val_1 = E29x_Get_Page_Register( PAGE0_SEL, 0x4a );
    temp_val_2 = ( ( temp_val_1 & 0xEF ) | ( 1 << 4 ) );
    E29x_Set_Page_Register( PAGE0_SEL, 0x4a, temp_val_2 );
    /* ùѹ룬η */
    tmp = E29x_Get_Page_Register( PAGE3_SEL, 0x06 );
    tmp = tmp & ( ~( 1 << 2 ) );
    tmp = tmp | ( 0 << 2 );
    E29x_Set_Page_Register( PAGE3_SEL, 0x06, tmp );
}

/*!
 * @brief ģʽ
 */
void E29x_SetReceive( void )
{
    uint8e_t temp_val_1;
    uint8e_t temp_val_2;
    uint8e_t tmp;
    /* лյ· */
    E29x_SetAntenna( ANTENNA_RXEN );
    /* RXVCO */
    temp_val_1 = E29x_Get_Page_Register( PAGE0_SEL, 0x4a );
    temp_val_2 = ( temp_val_1 & 0xEF ) ;
    E29x_Set_Page_Register( PAGE0_SEL, 0x4a, temp_val_2 );
    /* ùѹ룬 */
    tmp = E29x_Get_Page_Register( PAGE3_SEL, 0x06 );
    tmp = tmp & ( ~( 3 << 0 ) );
    tmp = tmp | ( 2 << 0 );
    E29x_Set_Page_Register( PAGE3_SEL, 0x06, tmp );
}

/*!
 * @brief ŵģʽ LBT(Listen Before Talk)
 */
void E29x_SetCAD( void )
{
    /* GPIO11Ϊģʽ */
    E29x_Set_GPIO_Output( MODULE_GPIO_CAD_IRQ );
    /* CADֵΪ0x12鷶Χ0x10~0x20 */
    E29x_Set_Page_Register( PAGE1_SEL, 0x0f, 0x12 );
}

/*!
 * @brief ˳ŵģʽ LBT(Listen Before Talk)
 */
void E29x_SetCAD_Off( void )
{
    uint8e_t tmpreg = 0;
    /* δ֪ */
    tmpreg = E29x_Get_Page_Register( PAGE0_SEL, 0x66 );
    tmpreg &= 0xf7;
    E29x_Set_Page_Register( PAGE0_SEL, 0x66, tmpreg );
    /* CADֵ */
    E29x_Set_Page_Register( PAGE1_SEL, 0x0f, 0x15 );
}

/*!
 * @brief ģ鿪ʼ׼
 *
 * @return 0
 * @note
 */
uint8e_t E29x_GoReceive( void )
{
    /* ģʽл: */
    E29x_SetStby();
    /* жϱʶ */
    E29x_ClearIRQ();
    /* лΪģʽ */
    E29x_SetReceive();
    /* лģʽ */
    E29x_SetRegister( 0x02, RF_MODE_RX );
    /* ״̬¼  Ѿʼգ״̬Ϊȴ״̬ */
    E29x_Status = GO_WAIT_RECEIVE;
    return OK;
}

/*!
 * @brief ģдݣʼߴ
 *
 * @param data ָݰ
 * @param size ݰ
 *
 * @return 0
 *
 * @note ݰ󳤶255
 */
uint8e_t E29x_GoTransmit( uint8e_t* data, uint8e_t size )
{
    uint8e_t irqStatus = 0;
    /* ģʽл: */
    E29x_SetStby();
    /* жϱʶ */
    E29x_ClearIRQ();
    /* лƵΪģʽ */
    E29x_SetTransmit();
    /* ÷ͳ */
    E29x_Set_Page_Register( PAGE1_SEL, 0x0C, size );
    /* лģʽ */
    E29x_SetRegister( 0x02, RF_MODE_TX );
    /* ䷢ */
    E29x_SetFIFO( data, size );
    /* ״̬¼  Ѿʼͣ״̬Ϊȴ */
    E29x_Status = GO_WAIT_TRANSMIT;
#if (EBYTE_TRANSMIT_MODE_BLOCKED)
    /*  ȴ */
    while( !( irqStatus & REG_IRQ_TX_DONE ) )
    {
        /* ƵоƬжϱ־λ */
        Ebyte_Port_DelayMs( 20 );
        irqStatus = E29x_GetIRQ();
    }
    /* ˷ ж */
    E29x_ClearIRQ();
    /*  */
    E29x_SetStby();
    /* صû */
    Ebyte_Port_TransmitCallback( 0x0001 );
#endif
    return OK;
}

/*!
 * @brief ģ(͹)
 *
 * @return 0
 */
uint8e_t E29x_GoSleep( void )
{
    E29x_SetSleep();
    return 0;
}

/*!
 * @brief 
 */
void E29x_TaskForIRQ( void )
{
}

/*!
 * @brief ѯ Ҫѭ
 *
 * @return 0
 * @note ȡڲʶλӶݽ
 */
uint8e_t E29x_TaskForPoll( void )
{
    uint8e_t irqStatus;
    /* ȡģڲжϱʶ״̬ */
    irqStatus = E29x_GetIRQ();
    /* 0ʾĳһλжλ */
    if( irqStatus != 0 )
    {
        /* ǽ */
        if( irqStatus & REG_IRQ_RX_DONE )
        {
            /* ģʽл: */
            E29x_SetStby();
            /* ȡ½ݵĳ  Ĵַ:0x7D */
            uint8e_t rxLength = E29x_Get_Page_Register( PAGE1_SEL, 0x7D );
            /* ȡ½ */
            E29x_GetFIFO( E29x_ReceiveBuffer, rxLength );
            /* صû */
            Ebyte_Port_ReceiveCallback( 0x0001, E29x_ReceiveBuffer, rxLength );
        }
        /* ж */
        E29x_ClearIRQ();
    }
    return 0;
}

/*!
 * @brief ȡģ
 *
 * @return ַָָ
 * @note ׼ַ ĩβн '\0'
 */
uint8e_t* E29x_GetName( void )
{
    return E29x_NameString;
}

/*!
 * @brief ȡģ汾
 *
 * @return 8λı
 * @note 0x10 V1.0
 */
uint8e_t E29x_GetDriverVersion( void )
{
    return  EBYTE_E29_PROGRAM_TYPE;
}

/*!
 * @brief ȡģ״̬
 *
 * @return ״̬
 *        δʼ     GO_INIT          =0x00
 *        л״̬ GO_BUSY          =0x01
 *        /    GO_STBY          =0x02
 *        ׼ջ GO_RECEIVE       =0x03
 *        ڼ GO_WAIT_RECEIVE  =0x04
 *        ׼ͻ GO_TRANSMIT      =0x05
 *        ȴ GO_WAIT_TRANSMIT =0x06
 *                 GO_SLEEP         =0x07
 *        ڲ     GO_ERROR         =0x08
 */
uint8e_t E29x_GetStatus( void )
{
    return ( uint8e_t )E29x_Status;;
}

/*!
 * @brief Ƶʼ
 */
static void E29x_RF_init( void )
{
    /* ϵ绽ƵоƬ */
    E29x_SetSleep_Wakeup();
    /* ״̬¼ */
    E29x_Status = GO_INIT;
    /* ƵؼĴʼ */
    E29x_Set_Page_Register( PAGE0_SEL, 0x06, 0x01 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x1f, 0x18 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x20, 0x15 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x23, 0x08 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x25, 0x08 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x27, 0x04 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x28, 0x14 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x2d, 0x33 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x2e, 0x33 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x2f, 0x33 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x30, 0x33 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x31, 0x33 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x32, 0x60 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x33, 0x07 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x3e, 0x2c );
    E29x_Set_Page_Register( PAGE0_SEL, 0x3c, 0xff );
    E29x_Set_Page_Register( PAGE0_SEL, 0x40, 0x50 );
    E29x_Set_Page_Register( PAGE0_SEL, 0x4c, 0xbf );
    E29x_Set_Page_Register( PAGE0_SEL, 0x4d, 0x0e );
    E29x_Set_Page_Register( PAGE0_SEL, 0x55, 0x02 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x0e, 0x44 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x0f, 0x0a );
    E29x_Set_Page_Register( PAGE1_SEL, 0x1e, 0x00 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x11, 0xA1 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x15, 0x38 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x2f, 0x0c );
    E29x_Set_Page_Register( PAGE1_SEL, 0x38, 0x02 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x57, 0x33 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x58, 0x33 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x59, 0x33 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x5a, 0x33 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x5b, 0x33 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x5c, 0x33 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x5d, 0x33 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x5e, 0x33 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x5f, 0x33 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x60, 0x33 );
    E29x_Set_Page_Register( PAGE1_SEL, 0x61, 0x11 );
    //E29x_Set_Page_Register(PAGE1_SEL, 0x67, 0xc0);
    E29x_Set_Page_Register( PAGE3_SEL, 0x06, 0x26 );
    E29x_Set_Page_Register( PAGE3_SEL, 0x10, 0x80 );
    E29x_Set_Page_Register( PAGE3_SEL, 0x11, 0x0d );
    E29x_Set_Page_Register( PAGE3_SEL, 0x12, 0x16 );
    E29x_Set_Page_Register( PAGE3_SEL, 0x18, 0xff );
    /* ʹAGC */
    E29x_Set_Page_Register( PAGE2_SEL, 0x06, 0x02 );
    /* AGCĴ */
    E29x_Set_Page_Register_Continue( PAGE2_SEL, 0x07, reg_agc_value, 98 );
    /* ƵؿƽΪ */
    E29x_Set_GPIO_Output( MODULE_GPIO_RX );
    E29x_Set_GPIO_Output( MODULE_GPIO_TX );
    /* Ƶжڲշ· */
    E29x_Set_GPIO_Status( MODULE_GPIO_RX, 0 );
    E29x_Set_GPIO_Status( MODULE_GPIO_TX, 0 );
    /* ״̬¼ */
    E29x_Status = GO_STBY;
}

/*!
 * @brief 趨ƵηΧ
 *
 * @param lo    Ƶ
 */
static void E29x_Set_LO_Freq( uint8e_t lo )
{
    uint8e_t reg_val = 0;
    /* ȡƵηΧĴ */
    reg_val = E29x_Get_Page_Register( PAGE0_SEL, 0x45 );
    reg_val &= ~( 0x03 );
    if( lo == LO_400M )
    {
        reg_val |= 0x02;
    }
    else
    {
        reg_val |= 0x01;
    }
    /* ƵηΧĴ */
    E29x_Set_Page_Register( PAGE0_SEL, 0x45, reg_val );
}

/*!
 * @brief 趨Ƶ
 *
 * @param freq  趨Ƶ
 * @return      ɱ־λ:OKɹ  FAILʧ
 */
static uint8e_t E29x_Set_Freq( uint32e_t freq )
{
    uint8e_t  reg_read;
    uint8e_t  reg_freq;
    uint32e_t tmp_var = 0;
    uint32e_t integer_part = 0;
    uint32e_t fractional_part = 0;
    uint8e_t  lowband_sel = 0;
    int fb, fc;
    if( ( freq >= 360000000 ) && ( freq <= 370000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0x0f )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 1;
        tmp_var = freq / 1000 * 25;
        integer_part = tmp_var / 100000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 100000;
        fc = ( fractional_part * 4 ) / 1000;
        E29x_Set_LO_Freq( LO_400M );
    }
    else if( ( freq >= 370000000 ) && ( freq <= 385000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0x2f )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 1;
        tmp_var = freq / 1000 * 25;
        integer_part = tmp_var / 100000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 100000;
        fc = ( fractional_part * 4 ) / 1000;
        E29x_Set_LO_Freq( LO_400M );
    }
    else if( ( freq >= 385000000 ) && ( freq <= 405000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0x4f )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 1;
        tmp_var = freq / 1000 * 25;
        integer_part = tmp_var / 100000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 100000;
        fc = ( fractional_part * 4 ) / 1000;
        E29x_Set_LO_Freq( LO_400M );
    }
    else if( ( freq > 405000000 ) && ( freq <= 430000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0x6f )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 1;
        tmp_var = freq / 1000 * 25;
        integer_part = tmp_var / 100000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 100000;
        fc = ( fractional_part * 4 ) / 1000;
        E29x_Set_LO_Freq( LO_400M );
    }
    else if( ( freq > 430000000 ) && ( freq <= 460000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0x8f )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 1;
        tmp_var = freq / 1000 * 25;
        integer_part = tmp_var / 100000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 100000;
        fc = ( fractional_part * 4 ) / 1000;
        E29x_Set_LO_Freq( LO_400M );
    }
    else if( ( freq > 460000000 ) && ( freq <= 495000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0xaf )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 1;
        tmp_var = freq / 1000 * 25;
        integer_part = tmp_var / 100000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 100000;
        fc = ( fractional_part * 4 ) / 1000;
        E29x_Set_LO_Freq( LO_400M );
    }
    else if( ( freq > 495000000 ) && ( freq <= 535000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0xcf )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 1;
        tmp_var = freq / 1000 * 25;
        integer_part = tmp_var / 100000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 100000;
        fc = ( fractional_part * 4 ) / 1000;
        E29x_Set_LO_Freq( LO_400M );
    }
    else if( ( freq > 535000000 ) && ( freq <= 600000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0xef )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 1;
        tmp_var = freq / 1000 * 25; //freq * 4 * 1.0 / 16000000,-5
        integer_part = tmp_var / 100000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 100000;
        fc = ( fractional_part * 4 ) / 1000; //lowband_sel = 1,1600/4,-5+2
        E29x_Set_LO_Freq( LO_400M );
    }
    else if( ( freq >= 720000000 ) && ( freq <= 740000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0x0f )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 0;
        tmp_var = freq / 1000 * 125;    //freq * 2 * 1.0 / 16000000,-6
        integer_part = tmp_var / 1000000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 1000000;
        fc = ( fractional_part * 8 ) / 10000; //lowband_sel = 0,1600/4,-6+2
        E29x_Set_LO_Freq( LO_800M );
    }
    else if( ( freq >= 740000000 ) && ( freq <= 770000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0x2f )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 0;
        tmp_var = freq / 1000 * 125;
        integer_part = tmp_var / 1000000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 1000000;
        fc = ( fractional_part * 8 ) / 10000;
        E29x_Set_LO_Freq( LO_800M );
    }
    else if( ( freq > 770000000 ) && ( freq <= 810000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0x4f )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 0;
        tmp_var = freq / 1000 * 125;
        integer_part = tmp_var / 1000000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 1000000;
        fc = ( fractional_part * 8 ) / 10000;
        E29x_Set_LO_Freq( LO_800M );
    }
    else if( ( freq > 810000000 ) && ( freq <= 860000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0x6f )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 0;
        tmp_var = freq / 1000 * 125;
        integer_part = tmp_var / 1000000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 1000000;
        fc = ( fractional_part * 8 ) / 10000;
        E29x_Set_LO_Freq( LO_800M );
    }
    else if( ( freq > 860000000 ) && ( freq <= 920000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0x8f )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 0;
        tmp_var = freq / 1000 * 125;
        integer_part = tmp_var / 1000000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 1000000;
        fc = ( fractional_part * 8 ) / 10000;
        E29x_Set_LO_Freq( LO_800M );
    }
    else if( ( freq > 920000000 ) && ( freq <= 990000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0xaf )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 0;
        tmp_var = freq / 1000 * 125;
        integer_part = tmp_var / 1000000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 1000000;
        fc = ( fractional_part * 8 ) / 10000;
        E29x_Set_LO_Freq( LO_800M );
    }
    else if( ( freq > 990000000 ) && ( freq <= 1070000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0xcf )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 0;
        tmp_var = freq / 1000 * 125;
        integer_part = tmp_var / 1000000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 1000000;
        fc = ( fractional_part * 8 ) / 10000;
        E29x_Set_LO_Freq( LO_800M );
    }
    else if( ( freq > 1070000000 ) && ( freq <= 1200000000 ) )
    {
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4a, 0xef )  != OK )
        {
            return FAIL;
        }
        lowband_sel = 0;
        tmp_var = freq / 1000 * 125;
        integer_part = tmp_var / 1000000;
        fb = integer_part - 20;
        fractional_part = tmp_var % 1000000;
        fc = ( fractional_part * 8 ) / 10000;
        E29x_Set_LO_Freq( LO_800M );
    }
    else
    {
        return FAIL;
    }
    if( ( freq >= 360000000 ) && ( freq <= 600000000 ) )
    {
        if( fc < 0xff )
        {
            fb = fb - 1;
            fc = fc + 400;
        }
    }
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x15, ( fb & 0x7F ) ) != OK )
    {
        return FAIL;
    }
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x16, ( fc & 0xff ) ) != OK )
    {
        return FAIL;
    }
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x17, ( ( fc >> 8 ) & 0x0f ) ) != OK )
    {
        return FAIL;
    }
    reg_read = E29x_Get_Page_Register( PAGE3_SEL, 0x18 );
    reg_read &= ~( ( 1 << 2 ) | ( 1 << 1 ) );
    reg_read |= ( 1 << 3 ) | ( lowband_sel << 2 ) | ( lowband_sel << 1 );
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x18, reg_read ) != OK )
    {
        return FAIL;
    }
    reg_freq = freq & 0xff;
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x09, reg_freq ) != OK )
    {
        return FAIL;
    }
    reg_freq = ( freq >> 8 ) & 0xff;
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x0a, reg_freq ) != OK )
    {
        return FAIL;
    }
    reg_freq = ( freq >> 16 ) & 0xff;
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x0b, reg_freq ) != OK )
    {
        return FAIL;
    }
    reg_freq = ( freq >> 24 ) & 0xff;
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x0c, reg_freq ) != OK )
    {
        return FAIL;
    }
    return OK;
}

/**
 * @brief set PA1_IB
 * @param[in] <mode> PA1_IB
 * @return result
 */
static uint8e_t E29x_Set_PA1_IB( uint8e_t mode )
{
    uint8e_t temp_val_1;
    uint8e_t temp_val_2;
    temp_val_1 = E29x_Get_Page_Register( PAGE0_SEL, 0x43 );
    temp_val_2 = ( ( temp_val_1 & 0xF0 ) | ( mode << 3 ) ) ;
    if( E29x_Set_Page_Register( PAGE0_SEL, 0x43, temp_val_2 ) != OK )
    {
        return FAIL;
    }
    else
    {
        return OK;
    }
}

/**
 * @brief set PA2_IB
 * @param[in] <mode> PA2_IB
 * @return result
 */
static uint8e_t E29x_Set_PA2_IB( uint8e_t mode )
{
    uint8e_t temp_val_1;
    uint8e_t temp_val_2;
    temp_val_1 = E29x_Get_Page_Register( PAGE0_SEL, 0x4B );
    temp_val_2 = ( ( temp_val_1 & 0xF0 ) | ( mode << 3 ) ) ;
    if( E29x_Set_Page_Register( PAGE0_SEL, 0x4B, temp_val_2 ) != OK )
    {
        return FAIL;
    }
    else
    {
        return OK;
    }
}

/**
 * @brief set LDO_TX_VSEL
 * @param[in] <mode> LDO_TX_VSEL
 * @return result
 */
static uint8e_t E29x_set_LDO_TX_VSEL( uint8e_t mode )
{
    uint8e_t temp_val_1;
    uint8e_t temp_val_2;
    temp_val_1 = E29x_Get_Page_Register( PAGE0_SEL, 0x35 );
    temp_val_2 = ( ( temp_val_1 & 0xF8 ) | mode ) ;
    if( E29x_Set_Page_Register( PAGE0_SEL, 0x35, temp_val_2 ) != OK )
    {
        return FAIL;
    }
    else
    {
        return OK;
    }
}

/*!
 * @brief 趨͹
 *
 * @param tx_power  趨ķ͹ʣֵ30Ӧ22 dBm
 * @return          ɱ־λ:OKɹ  FAILʧ
 */
uint8e_t E29_Set_Tx_Power( uint8e_t tx_power )
{
    if( tx_power == 0 )
    {
        if( E29x_Set_Page_Register( PAGE1_SEL, 0x63, 0 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA1_IB( 0 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA2_IB( 0 ) != OK )
        {
            return FAIL;
        }
        if( E29x_set_LDO_TX_VSEL( 0 ) != OK )
        {
            return FAIL;
        }
    }
    else if( tx_power == 1 )
    {
        if( E29x_Set_Page_Register( PAGE1_SEL, 0x63, 0 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA1_IB( 0 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA2_IB( 0 ) != OK )
        {
            return FAIL;
        }
        if( E29x_set_LDO_TX_VSEL( 3 ) != OK )
        {
            return FAIL;
        }
    }
    else if( tx_power == 2 )
    {
        if( E29x_Set_Page_Register( PAGE1_SEL, 0x63, 0 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA1_IB( 0 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA2_IB( 1 ) != OK )
        {
            return FAIL;
        }
        if( E29x_set_LDO_TX_VSEL( 3 ) != OK )
        {
            return FAIL;
        }
    }
    else if( ( tx_power > 2 ) && ( tx_power < 11 ) )
    {
        if( E29x_Set_Page_Register( PAGE1_SEL, 0x63, tx_power - 3 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA1_IB( 1 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA2_IB( 1 ) != OK )
        {
            return FAIL;
        }
        if( E29x_set_LDO_TX_VSEL( 3 ) != OK )
        {
            return FAIL;
        }
    }
    else if( ( tx_power > 10 ) && ( tx_power < 25 ) )
    {
        if( E29x_Set_Page_Register( PAGE1_SEL, 0x63, ( ( ( tx_power - 10 ) << 4 ) | 0x07 ) ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA1_IB( 1 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA2_IB( 1 ) != OK )
        {
            return FAIL;
        }
        if( E29x_set_LDO_TX_VSEL( 3 ) != OK )
        {
            return FAIL;
        }
    }
    else if( tx_power == 25 )
    {
        if( E29x_Set_Page_Register( PAGE1_SEL, 0x63, 0xE7 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x35, 0x1F ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x43, 0xF1 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_Page_Register( PAGE0_SEL, 0x4B, 0x4F ) != OK )
        {
            return FAIL;
        }
    }
    else if( tx_power == 26 )
    {
        if( E29x_Set_Page_Register( PAGE1_SEL, 0x63, ( ( tx_power - 11 ) << 4 ) | 0x07 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA1_IB( 1 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA2_IB( 1 ) != OK )
        {
            return FAIL;
        }
        if( E29x_set_LDO_TX_VSEL( 3 ) != OK )
        {
            return FAIL;
        }
    }
    else if( ( tx_power > 26 ) && ( tx_power < 31 ) )
    {
        if( E29x_Set_Page_Register( PAGE1_SEL, 0x63, 0xf7 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA1_IB( 1 ) != OK )
        {
            return FAIL;
        }
        if( E29x_Set_PA2_IB( 1 ) != OK )
        {
            return FAIL;
        }
        if( E29x_set_LDO_TX_VSEL( tx_power - 23 ) != OK )
        {
            return FAIL;
        }
    }
    else
    {
        return FAIL;
    }
    return OK;
}


/*!
 * @brief زģʽ
 * @param freq  趨ķƵ
 */
void E29x_Set_Carrier_Wave_freq( uint32e_t freq )
{
    uint8e_t buf[1] = {0};
    /* 趨Ƶ */
    E29x_Set_Freq( freq );
    /* ģʽл: */
    E29x_SetStby();
    /* жϱʶ */
    E29x_ClearIRQ();
    /* лƵΪģʽ */
    E29x_SetTransmit();
    /* ÷ͳ */
    E29x_Set_Page_Register( PAGE1_SEL, 0x0C, 1 );
    /* лģʽ */
    E29x_SetRegister( 0x02, RF_MODE_TX );
    /* ䷢ */
    E29x_SetFIFO( buf, 1 );
    /* ״̬¼  Ѿʼͣ״̬Ϊȴ */
    E29x_Status = GO_WAIT_TRANSMIT;
}

/*!
 * @brief ˳زģʽ
 */
void E29x_Stop_Carrier_Wave( void )
{
    uint8e_t temp_val_1;
    uint8e_t temp_val_2;
    /* лģʽΪSTB3 */
    E29x_SetStby();
    /* Ƶز */
    temp_val_1 = E29x_Get_Page_Register( PAGE1_SEL, 0x1e );
    temp_val_2 = ( ( temp_val_1 & 0xFE ) | ( 0 << 0 ) ) ;
    E29x_Set_Page_Register( PAGE1_SEL, 0x1e, temp_val_2 );
}

/*!
 * @brief Ƶʼ
 *
 * @param frequency         λHz   433000000 ʾ433.0MHz
 * @param bandwidth         ԤƵ
 * @param spreading_factor  ԤƵ
 * @param coding_rate       Ԥ
 * @param crc               CRCУ鿪
 * @param preamble_length   ǰ볤
 * @param sync_word         ͬ
 * @param output_power      
 * @param low_datarate      ͿŻ
 *
 * @return 0:  1:ò
 *
 * @note Ԥebyte_E29x.hеĺ궨  E29X_LORA_BANDWIDTH
 */
uint8e_t E29x_Set_RF_Init( uint32e_t frequency, uint8e_t bandwidth, uint8e_t spreading_factor, uint8e_t coding_rate,
                           uint8e_t crc, uint16e_t preamble_length, uint8e_t sync_word, int8e_t output_power, uint8e_t low_datarate )
{
    uint8e_t reg_value;
    uint8e_t temp_val_1;
    uint8e_t temp_val_2;
    /* лģʽΪSTB3 */
    E29x_SetStby();
    /* 趨Ƶ */
    if( E29x_Set_Freq( frequency ) != OK )
    {
        return FAIL;
    }
    /* Ƶ  */
    temp_val_1 = E29x_Get_Page_Register( PAGE3_SEL, 0x0d );
    temp_val_2 = ( ( temp_val_1 & 0x0F ) | ( bandwidth << 4 ) ) ;
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x0d, temp_val_2 ) != OK )
    {
        return FAIL;
    }
    /* 趨ƵSF */
    if( spreading_factor < 7 || spreading_factor > 9 )
    {
        return FAIL;
    }
    else
    {
        temp_val_1 = E29x_Get_Page_Register( PAGE3_SEL, 0x0e );
        temp_val_2 = ( ( temp_val_1 & 0x0F ) | ( spreading_factor << 4 ) );
        if( E29x_Set_Page_Register( PAGE3_SEL, 0x0e, temp_val_2 ) != OK )
        {
            return FAIL;
        }
    }
    /* ñ CR */
    reg_value = E29x_Get_Page_Register( PAGE3_SEL, 0x0d );
    reg_value &= ~( 0x7 << 1 );
    reg_value |= ( coding_rate << 1 );
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x0d, reg_value ) != OK )
    {
        return FAIL;
    }
    /* ݰУ CRC */
    temp_val_1 = E29x_Get_Page_Register( PAGE3_SEL, 0x0e );
    temp_val_2 = ( ( temp_val_1 & 0xF7 ) | ( crc << 3 ) ) ;
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x0e, temp_val_2 ) != OK )
    {
        return FAIL;
    }
    /* ǰ볤 */
    temp_val_1 = ( preamble_length )      & 0xff;
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x13, temp_val_1 )  != OK )
    {
        return FAIL;
    }
    temp_val_2 = ( preamble_length >> 8 ) & 0xff;
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x14, temp_val_2 )  != OK )
    {
        return FAIL;
    }
    /* ͬ */
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x0f, sync_word ) != OK )
    {
        return FAIL;
    }
    /*  */
    E29_Set_Tx_Power( output_power );
    /* õͿ */
    temp_val_1 = E29x_Get_Page_Register( PAGE3_SEL, 0x12 );
    temp_val_2 = ( ( temp_val_1 & 0xF7 ) | ( low_datarate << 3 ) ) ;
    if( E29x_Set_Page_Register( PAGE3_SEL, 0x12, temp_val_2 ) != OK )
    {
        return FAIL;
    }
    return OK;
}


/*!
 * @brief BBYTE ģʼ
 */
uint8e_t E29x_Init( void )
{
    uint8e_t result;
    /* Ƶʼ */
    E29x_RF_init();
    /* ִLoRaģʽʼ ü궨 */
    result = E29x_Set_RF_Init( E29X_FREQUENCY_START,         //زƵ
                               E29X_LORA_BANDWIDTH,         //Ƶ
                               E29X_LORA_SPREADING_FACTOR,  //Ƶ
                               E29X_LORA_CORING_RATE,       //
                               E29X_LORA_CRC,               //CRC
                               E29X_LORA_PREAMBLE_LENGTH,   //ǰ볤
                               E29X_LORA_SYNC_WORD,         //ͬ
                               E29X_OUTPUT_POWER,           //
                               E29X_LORA_LOW_DATARATE );    //ͿŻ
    /* ģʽ */
    E29x_GoReceive();
    return result;
}