/**
  **********************************************************************************
  * @file      ebyte_e32x.c
  * @brief     EBYTE E32 (sx1276/8) ϵ
  * @details   μ https://www.ebyte.com/
  * @author    JiangHeng
  * @date      2021-09-10
  * @version   1.0.0
  **********************************************************************************
  * @copyright BSD License
  *            ɶڰصӿƼ޹˾
  *   ______   ____   __     __  _______   ______
  *  |  ____| |  _ \  \ \   / / |__   __| |  ____|
  *  | |__    | |_) |  \ \_/ /     | |    | |__
  *  |  __|   |  _ <    \   /      | |    |  __|
  *  | |____  | |_) |    | |       | |    | |____
  *  |______| |____/     |_|       |_|    |______|
  *
  **********************************************************************************
  */

#include "ebyte_e32x.h"

/// ʶ룺ģ
#if defined(EBYTE_E32_400M20S)
static uint8e_t E32x_NameString[] = "E32-400M20S";
#elif defined(EBYTE_E32_900M20S)
static uint8e_t E32x_NameString[] = "E32-900M20S";
#endif

/// ʶ룺汾
#define EBYTE_E32_PROGRAM_TYPE 0x10

/// ģڲշ·ģʽ
#define ANTENNA_RXEN  0
#define ANTENNA_TXEN  1

/// Ƶʸ
#define FREQ_STEP     61.03515625

/// жϼĴʶ ֵ֧
#define IRQ_RXTIMEOUT                     0x80
#define IRQ_RXDONE                        0x40
#define IRQ_PAYLOADCRCERROR               0x20
#define IRQ_VALIDHEADER                   0x10
#define IRQ_TXDONE                        0x08
#define IRQ_CADDONE                       0x04
#define IRQ_FHSSCHANGEDCHANNEL            0x02
#define IRQ_CADDETECTED                   0x01

typedef enum
{
    GO_INIT          = 0x00,
    GO_BUSY          = 0x01,
    GO_STBY          = 0x02,
    GO_RECEIVE       = 0x03,
    GO_WAIT_RECEIVE  = 0x04,
    GO_TRANSMIT      = 0x05,
    GO_WAIT_TRANSMIT = 0x06,
    GO_SLEEP         = 0x07,
    GO_ERROR         = 0x08
} E32x_Status_t;

/// ״̬ʶ
static E32x_Status_t E32x_Status = GO_INIT;

/// ݻ
static uint8e_t E32x_ReceiveBuffer[255] ;



/*!
 * @brief лģڲշ·
 *
 * @param mode лָģʽ
 *      @arg ANTENNA_RXEN: ģʽ
 *      @arg ANTENNA_TXEN: ģʽ
 *
 * @note շ·лȷᵼ·书½ ޷շ
 */
static void E32X_SetAntenna( uint8e_t mode )
{
    if( mode == ANTENNA_TXEN )
    {
        Ebyte_Port_RxenIoControl( 0 );
        Ebyte_Port_TxenIoControl( 1 );
    }
    else
    {
        Ebyte_Port_TxenIoControl( 0 );
        Ebyte_Port_RxenIoControl( 1 );
    }
}

/*!
 * @brief дĴ
 *
 * @param address Ĵʼַ
 * @param data    дָ
 * @param size    Ĵ
 */
static void E32x_SetRegisters( uint8e_t address, uint8e_t* data, uint8e_t size )
{
    uint8e_t i;
    /* CSƬѡ ͵ƽѡ */
    Ebyte_Port_SpiCsIoControl( 0 );
    /* Ĵʼַ */
    Ebyte_Port_SpiTransmitAndReceivce( address | 0x80 ); //bit7=1 ʾݰΪд  |0x80
    /* д */
    for( i = 0; i < size; i++ )
    {
        Ebyte_Port_SpiTransmitAndReceivce( data[i] );
    }
    /* CSƬѡ ߵƽ */
    Ebyte_Port_SpiCsIoControl( 1 );
}

/*!
 * @brief ȡĴ
 *
 * @param address Ĵʼַ
 * @param data    ָȡݴŻ
 * @param size    Ĵ
 */
static void E32x_GetRegisters( uint8e_t address, uint8e_t* data, uint8e_t size )
{
    uint8e_t i;
    /* CSƬѡ ͵ƽѡ */
    Ebyte_Port_SpiCsIoControl( 0 );
    /* Ĵʼַ */
    Ebyte_Port_SpiTransmitAndReceivce( address & 0x7F ); //bit7=0 ʾݰΪȡ  &0x7F
    /* ȡ */
    for( i = 0; i < size; i++ )
    {
        data[i] = Ebyte_Port_SpiTransmitAndReceivce( 0 );
    }
    /* CSƬѡ ߵƽ */
    Ebyte_Port_SpiCsIoControl( 1 );
}

/*!
 * @brief д뵥Ĵ
 *
 * @param address Ĵʼַ
 * @param data    д
 */
static void E32x_SetRegister( uint8e_t address, uint8e_t data )
{
    E32x_SetRegisters( address, &data, 1 );
}

/*!
 * @brief ȡĴ
 *
 * @param address Ĵʼַ
 * @param data    д
 */
static uint8e_t E32x_GetRegister( uint8e_t address )
{
    uint8e_t data;
    E32x_GetRegisters( address, &data, 1 );
    return data;
}

/*!
 * @brief дģڲݶ
 *
 * @param data ָݰ
 * @param size дݳ
 *
 * @note ģлΪģʽ󣬻ԶӶȡݽߴ
 */
static void E32x_SetFIFO( uint8e_t* data, uint8e_t size )
{
    E32x_SetRegisters( 0, data, size );
}

/*!
 * @brief ȡģڲݶ
 *
 * @param data ָûݻ
 * @param size ݳ
 *
 * @note עʱ򣬽ģʽʱҪöȡַλȡʵʽճȣɺٶȡFIFO
 */
static void E32x_GetFIFO( uint8e_t* data, uint8e_t size )
{
    E32x_GetRegisters( 0, data, size );
}

/*!
 * @brief ģ鸴λ
 *
 * @note ڲĴỹԭΪĬֵ
 */
static void E32x_Reset( void )
{
    Ebyte_Port_DelayMs( 10 );
    Ebyte_Port_RstIoControl( 0 );
    Ebyte_Port_DelayMs( 20 );
    Ebyte_Port_RstIoControl( 1 );
    Ebyte_Port_DelayMs( 20 );
}

/*!
 * @brief ģGPIO(DIO)Ĺ
 *
 * @param regDioMapping1  Ĵַ:0x40
 * @param regDioMapping2  Ĵַ:0x41
 *
 * @note һָһڽջ߷ʱMCUж
 *       ܹӦжƵ SX1278ֲ(2.0.1 Digital IO Pin Mapping)
 */
void E32x_SetGPIO( uint8e_t regDioMapping1, uint8e_t regDioMapping2 )
{
    E32x_SetRegister( 0x40, regDioMapping1 );
    E32x_SetRegister( 0x41, regDioMapping2 );
}

/*!
 * @brief жģǷ
 *
 * @note ͨȡоƬ汾ʵֵ
 */
uint8e_t E32x_IsExist( void )
{
    uint8e_t result = 1;
    uint8e_t version;
    /* ȡоƬ汾 Ӧ0x12 Ĵַ:0x42   */
    version = E32x_GetRegister( 0x42 );
    if( version == 0x12 )
    {
        /*  */
        result = 0;
    }
    return result;
}

/*!
 * @brief ж
 *
 * @param irq ж 궨 λ
 *      @arg IRQ_RXTIMEOUT
 *      @arg IRQ_RXDONE
 *      @arg IRQ_PAYLOADCRCERROR
 *      @arg IRQ_VALIDHEADER
 *      @arg IRQ_TXDONE
 *      @arg IRQ_CADDONE
 *      @arg IRQ_FHSSCHANGEDCHANNEL
 *      @arg IRQ_CADDETECTED
 */
void E32x_SetIRQ( uint8e_t irq )
{
    uint8e_t reg_value;
    /* жϵĻΪӦΪ 0 */
    reg_value = ~irq;
    E32x_SetRegister( 0x11, reg_value );
}

/*!
 * @brief ȡжϱʶ
 *
 * @return һ㶼Ƿ/
 *       0x80:IRQ_RXTIMEOUT
 *       0x40:IRQ_RXDONE
 *       0x20:IRQ_PAYLOADCRCERROR
 *       0x10:IRQ_VALIDHEADER
 *       0x08:IRQ_TXDONE
 *       0x04:IRQ_CADDONE
 *       0x02:IRQ_FHSSCHANGEDCHANNEL
 *       0x01:IRQ_CADDETECTED
 */
uint8e_t E32x_GetIRQ( void )
{
    /* ȡж״̬ Ĵַ:0x12 */
    return E32x_GetRegister( 0x12 );
}

/*!
 * @brief жϱʶ
 */
void E32x_ClearIRQ( void )
{
    uint8e_t reg_value = 0;
    /* ȡж״̬ Ĵַ:0x12 */
    reg_value = E32x_GetRegister( 0x12 );
    /* жΪдӦλ 1 */
    E32x_SetRegister( 0x12, reg_value );
}

/*!
 * @brief ģʽ
 */
void E32x_SetSleep( void )
{
    uint8e_t result;
    /* ȡģʽƼĴ Ĵַ:0x01   */
    result = E32x_GetRegister( 0x01 );
    /* bit2bit0 3λʾģʽ 0SLEEPģʽ  */
    result &= 0xF8;//bit
    /* д */
    E32x_SetRegister( 0x01, result );
    /* ״̬¼ */
    E32x_Status = GO_SLEEP;
}

/*!
 * @brief ģʽ
 */
void E32x_SetStby( void )
{
    uint8e_t result;
    /* ȡģʽƼĴ Ĵַ:0x01   */
    result = E32x_GetRegister( 0x01 );
    /* bit2bit0 3λʾģʽ 0x01STDBYģʽ  */
    result &= 0xF8;//bit
    result |= 0x01;//bitλ
    /* д */
    E32x_SetRegister( 0x01, result );
    /* ״̬¼ */
    E32x_Status = GO_STBY;
}

/*!
 * @brief 뷢ģʽ
 */
void E32x_SetTransmit( void )
{
    uint8e_t result;
    /* ȡģʽƼĴ Ĵַ:0x01   */
    result = E32x_GetRegister( 0x01 );
    /* bit2bit0 3λʾģʽ 0x03TXģʽ  */
    result &= 0xF8;//bit
    result |= 0x03;//bitλ
    /* д */
    E32x_SetRegister( 0x01, result );
    /* ״̬¼  Ѿʼͣ״̬Ϊȴ */
    E32x_Status = GO_WAIT_TRANSMIT;
}

/*!
 * @brief ģʽ
 */
void E32x_SetReceive( void )
{
    uint8e_t result;
    /* ȡģʽƼĴ Ĵַ:0x01   */
    result = E32x_GetRegister( 0x01 );
    /* bit2bit0 3λʾģʽ 0x05RXģʽ  */
    result &= 0xF8;//bit
    result |= 0x05;//bitλ
    /* д */
    E32x_SetRegister( 0x01, result );
    /* ״̬¼  Ѿʼգ״̬Ϊȴ״̬ */
    E32x_Status = GO_WAIT_RECEIVE;
}

/*!
 * @brief ŵģʽ LBT(Listen Before Talk)
 */
void E32x_SetCAD( void )
{
    uint8e_t result;
    /* ȡģʽƼĴ Ĵַ:0x01   */
    result = E32x_GetRegister( 0x01 );
    /* bit2bit0 3λʾģʽ 0x07CADģʽ  */
    result &= 0xF8;//bit
    result |= 0x07;//bitλ
    /* д */
    E32x_SetRegister( 0x01, result );
}

/*!
 * @brief LoRaģʽʼ
 *
 * @param frequency         λHz   433000000 ʾ433.0MHz
 * @param bandwidth         ԤƵ
 * @param spreading_factor  ԤƵ
 * @param coding_rate       Ԥ
 * @param crc               CRCУ鿪
 * @param preamble_length   ǰ볤
 * @param sync_word         ͬ
 * @param output_power       (:1dBm)
 * @param low_datarate      ͿŻ
 *
 * @return 0:  1:ò
 *
 * @note Ԥebyte_E32x.hеĺ궨  E32X_LORA_BANDWIDTH
 */
uint8e_t E32x_SetLoRaInit( uint32e_t frequency, uint8e_t bandwidth, uint8e_t spreading_factor, uint8e_t coding_rate,
                           uint8e_t crc, uint16e_t preamble_length, uint8e_t sync_word, int8e_t output_power, uint8e_t low_datarate )
{
    uint8e_t config[8];
    uint8e_t reg_value;
    uint32e_t freq;
    /* ֻSleepģʽ лƷʽ */
    E32x_SetSleep();
    /* лΪLoRaģʽ Ĵַ:0x01 bit7=1 */
    E32x_SetRegister( 0x01, 0x80 );
    /*  */
    E32x_SetStby();
    /* Ƶ */
    freq = ( uint32e_t )( ( double )frequency / ( double )FREQ_STEP );
    config[0] = ( uint8e_t )( ( freq >> 16 ) & 0xFF );
    config[1] = ( uint8e_t )( ( freq >> 8 ) & 0xFF );
    config[2] = ( uint8e_t )( freq & 0xFF );
    E32x_SetRegisters( 0x06, config, 3 );  //Ĵʼַ:0x06  3ֽ
    /* Ƶ SF
       SF = 6  LoRa ƽʵֵʴҪ */
    if( spreading_factor < 6 || spreading_factor > 12 )
    {
        return 1;    //
    }
    if( spreading_factor == 6 )
    {
        reg_value = E32x_GetRegister( 0x31 );   //DetectionOptimize Ĵַ:0x31 bit2bit0
        reg_value = ( reg_value & 0xF8 ) | 0x05;//0x05רSF6
        E32x_SetRegister( 0x31, reg_value );
    }
    else
    {
        reg_value = E32x_GetRegister( 0x31 );   //DetectionOptimize Ĵַ:0x31 bit2bit0
        reg_value = ( reg_value & 0xF8 ) | 0x03;//0x03רSF7SF12
        E32x_SetRegister( 0x31, reg_value );
    }
    reg_value = E32x_GetRegister( 0x1E );       //SpreadingFactor Ĵַ:0x1E bit7bit4
    reg_value = ( reg_value & 0x0F ) | ( spreading_factor << 4 );
    E32x_SetRegister( 0x1E, reg_value );
    /* ñ CR */
    if( coding_rate < 1 || coding_rate > 4 )
    {
        return 1;    //
    }
    reg_value = E32x_GetRegister( 0x1D );       //CodingRate Ĵַ:0x1D bit3bit1
    reg_value = ( reg_value & 0xF1 ) | ( coding_rate << 1 );
    E32x_SetRegister( 0x1D, reg_value );
    /* ݰУ CRC */
    if( crc > 1 )
    {
        return 1;    //
    }
    reg_value = E32x_GetRegister( 0x1E );       //RxPayloadCrcOn Ĵַ:0x1E bit7bit4
    reg_value = ( reg_value & 0xFB ) | ( crc << 2 );
    E32x_SetRegister( 0x1E, reg_value );
    /* Ƶ  */
    if( bandwidth > 9 )
    {
        return 1;    //
    }
    reg_value = E32x_GetRegister( 0x1D );       //Bw  Ĵַ:0x1D bit7bit4
    reg_value = ( reg_value & 0x0F ) | ( bandwidth << 4 );
    E32x_SetRegister( 0x1D, reg_value );
    /* ǰ볤 */
    config[0] = ( preamble_length >> 8 ) & 0x00FF;
    config[1] = preamble_length & 0xFF;
    E32x_SetRegisters( 0x20, config, 2 ); //Ĵʼַ:0x20  2ֽ
    /* ͬ */
    E32x_SetRegister( 0x39, sync_word ); //Ĵʼַ:0x39
    /* õͿŻ */
    if( low_datarate > 1 )
    {
        return 1;    //
    }
    reg_value = E32x_GetRegister( 0x26 );       //LowDataRateOptimize Ĵַ:0x26 bit3
    reg_value = ( reg_value & 0xF7 ) | ( low_datarate << 3 );
    E32x_SetRegister( 0x26, reg_value );
    /*  ͹ѡ2-17Ϊ лΪ17-20 */
    if( output_power < 2 || output_power > 20 )
    {
        return 1;    //
    }
    if( output_power >= 17 ) // 17-20 dBm
    {
        E32x_SetRegister( 0x4D, 0x87 );          //RegPaDac Ĵַ:0x4D  ģɿʷΧлΪ 5dBm20dBm
        reg_value  = E32x_GetRegister( 0x09 );   //PaSelect Ĵַ:0x09
        reg_value |= 0x80 ;                      //bit7  PA_BOOST
        reg_value |= 0x70 ;                      //MaxPower
        reg_value = ( reg_value & 0xF0 ) | ( ( uint8e_t )( ( uint16e_t )( output_power - 5 ) & 0x0F ) ); //OutputPower
        E32x_SetRegister( 0x09, reg_value );
        E32x_SetRegister( 0x0B, 0x3B );          //RegOcp   Ĵַ:0x0B   
    }
    else                  // 2-17 dBm
    {
        E32x_SetRegister( 0x4D, 0x84 );          //RegPaDac Ĵַ:0x4D  ģɿʷΧлΪ 2dBm17dBm
        reg_value  = E32x_GetRegister( 0x09 );   //PaSelect Ĵַ:0x09
        reg_value &= 0x7F ;                      //bit7  PA_BOOST
        reg_value |= 0x70 ;                      //MaxPower
        reg_value = ( reg_value & 0xF0 ) | ( ( uint8e_t )( ( uint16e_t )( output_power - 2 ) & 0x0F ) ); //OutputPower
        E32x_SetRegister( 0x09, reg_value );
    }
    return 0;
}

/*!
 * @brief ģ鿪ʼ׼
 *
 * @return 0
 * @note
 */
uint8e_t E32x_GoReceive( void )
{
    /* ģʽл: */
    E32x_SetStby();
    /* رƵ Ĵַ:0x24 */
    E32x_SetRegister( 0x24, 0 );
    /* GPIOӳ DIO0ӳ䵽RxDone ж
       ̲ѯȡģڲ־λķʽжǷɡԣIOҲûõ
       迼жϣҪIOжϺжȡݣжϼ */
    E32x_SetGPIO( 0, 0 );
    /* FIFO: ýݴʼַ 0ʼ  Ĵַ:0x0F */
    E32x_SetRegister( 0x0F, 0 );
    /* IO: յ· */
    E32X_SetAntenna( ANTENNA_RXEN );
    /* ж */
    E32x_SetIRQ( IRQ_RXDONE );
    /* жϱʶ */
    E32x_ClearIRQ();
    /* лΪģʽ */
    E32x_SetReceive();
    return 0;
}

/*!
 * @brief ģдݣʼߴ
 *
 * @param data ָݰ
 * @param size ݰ
 *
 * @return 0
 *
 * @note ݰ󳤶255
 */
uint8e_t E32x_GoTransmit( uint8e_t* data, uint8e_t size )
{
    uint8e_t irqStatus = 0;
    /* ģʽл: */
    E32x_SetStby();
    /* رƵ Ĵַ:0x24 */
    E32x_SetRegister( 0x24, 0 );
    /* ݰ Ĵַ:0x22 */
    E32x_SetRegister( 0x22, size );
    /* FIFO: ͵ַ128ʼ  Ĵַ:0x0E */
    E32x_SetRegister( 0x0E, 0x80 );
    /* FIFO: SPIдַ128ʼ  Ĵַ:0x0D */
    E32x_SetRegister( 0x0D, 0x80 );
    /* FIFO:д뷢 */
    E32x_SetFIFO( data, size );
    /* GPIOӳ DIO0ӳ䵽TxDone ж
       ̲ѯȡģڲ־λķʽжǷɡԣIOҲûõ
       迼жϣҪú궨 EBYTE_TRANSMIT_MODE_BLOCKED IOжϴ */
    E32x_SetGPIO( 0x40, 0x40 );
    /* IO: ͵· */
    E32X_SetAntenna( ANTENNA_TXEN );
    /* ж */
    E32x_SetIRQ( IRQ_TXDONE );
    /* жϱʶ */
    E32x_ClearIRQ();
    /* лΪģʽ */
    E32x_SetTransmit();
#if (EBYTE_TRANSMIT_MODE_BLOCKED)
    /*  ȴ */
    do
    {
        Ebyte_Port_DelayMs( 1 );
        irqStatus = E32x_GetIRQ();
    }
    while( irqStatus == 0xFF || ( !( irqStatus & IRQ_TXDONE ) ) ); //ڸʱ,иʶ0xFF,ΪǴ
    /* ˷ ж */
    E32x_ClearIRQ();
    /*  */
    E32x_SetStby();
    /* صû */
    Ebyte_Port_TransmitCallback( 0x0001 );
#endif
    return 0;
}

/*!
 * @brief ģ(͹)
 *
 * @return 0
 */
uint8e_t E32x_GoSleep( void )
{
    E32x_SetSleep();
    return 0;
}


/*!
 * @brief BBYTE ģʼ
 */
uint8e_t E32x_Init( void )
{
    uint8e_t result;
    /* λ */
    E32x_Reset();
    /* 飺ģǷ */
    result = E32x_IsExist();
    if( result != 0 )
    {
        return 1;
    }
    /* ʼLoRa  FSK */
    switch( E32X_MODULATION_TYPE )
    {
        case 0:
            /* ִLoRaģʽʼ ü궨 */
            E32x_SetLoRaInit( E32X_FREQUENCY_START,        //زƵ
                              E32X_LORA_BANDWIDTH,         //Ƶ
                              E32X_LORA_SPREADING_FACTOR,  //Ƶ
                              E32X_LORA_CORING_RATE,       //
                              E32X_LORA_CRC,               //CRC
                              E32X_LORA_PREAMBLE_LENGTH,   //ǰ볤
                              E32X_LORA_SYNC_WORD,         //ͬ
                              E32X_OUTPUT_POWER,           //
                              E32X_LORA_LOW_DATARATE );    //ͿŻ
            break;
        case 1:
            //FSK
            break;
        default:
            break;
    }
    E32x_GoReceive();
    return 0;
}


/*!
 * @brief 
 */
void E32x_TaskForIRQ( void )
{
}

/*!
 * @brief ѯ Ҫѭ
 *
 * @return 0
 * @note ȡڲʶλӶݽ
 */
uint8e_t E32x_TaskForPoll( void )
{
    uint8e_t irqStatus;
    /* ȡģڲжϱʶ״̬ */
    irqStatus = E32x_GetIRQ();
    /* 0ʾĳһλжλ */
    if( irqStatus != 0 )
    {
        /* ǽ */
        if( irqStatus & IRQ_RXDONE )
        {
            /* ģʽл: */
            E32x_SetStby();
            /* ȡ½ݴFIFOеλ  Ĵַ:0x10 */
            uint8e_t rxAddress = E32x_GetRegister( 0x10 );
            /* ȡ½ݵĳ  Ĵַ:0x13 */
            uint8e_t rxLength = E32x_GetRegister( 0x13 );
            /* SPIȡFIFO ʼַ  Ĵַ:0x0D */
            E32x_SetRegister( 0x0D, rxAddress );
            /* ȡFIFO */
            E32x_GetFIFO( E32x_ReceiveBuffer, 100 );
            /* صû */
            Ebyte_Port_ReceiveCallback( 0x0001, E32x_ReceiveBuffer, rxLength );
        }
        /* ж */
        E32x_ClearIRQ();
    }
    return 0;
}

/*!
 * @brief ȡģ
 *
 * @return ַָָ
 * @note ׼ַ ĩβн '\0'
 */
uint8e_t* E32x_GetName( void )
{
    return E32x_NameString;
}

/*!
 * @brief ȡģ汾
 *
 * @return 8λı
 * @note 0x10 V1.0
 */
uint8e_t E32x_GetDriverVersion( void )
{
    return  EBYTE_E32_PROGRAM_TYPE;
}

/*!
 * @brief ȡģ״̬
 *
 * @return ״̬
 *        δʼ     GO_INIT          =0x00
 *        л״̬ GO_BUSY          =0x01
 *        /    GO_STBY          =0x02
 *        ׼ջ GO_RECEIVE       =0x03
 *        ڼ GO_WAIT_RECEIVE  =0x04
 *        ׼ͻ GO_TRANSMIT      =0x05
 *        ȴ GO_WAIT_TRANSMIT =0x06
 *                 GO_SLEEP         =0x07
 *        ڲ     GO_ERROR         =0x08
 */
uint8e_t E32x_GetStatus( void )
{
    return ( uint8e_t )E32x_Status;;
}
