#include "ebyte_e49x.h"

/* ʶ汾 */
#define EBYTE_E49_PROGRAM_TYPE 0x10 

/// ʶģ
#if defined(EBYTE_E49_400M20S)
static uint8e_t E49x_NameString[] = "E49-400M20S";
#elif defined(EBYTE_E49_900M20S)
static uint8e_t E49x_NameString[] = "E49-900M20S";
#endif 

#define  GPIO1_DOUT          0x00
#define  GPIO1_DIN           0x00
#define  GPIO1_IRQ1          0x01
#define  GPIO1_IRQ2          0x02
#define  GPIO1_DCLK          0x03

#define  GPIO2_IRQ1          0x00
#define  GPIO2_IRQ2          0x04
#define  GPIO2_DOUT          0x08
#define  GPIO2_DIN           0x08
#define  GPIO2_DCLK          0x0C

#define  GPIO3_CLKO          0x00
#define  GPIO3_DOUT          0x10
#define  GPIO3_DIN           0x10
#define  GPIO3_IRQ2          0x20
#define  GPIO3_DCLK          0x30

#define  FIFO_READ           0x00
#define  FIFO_WRITE          0x01   

#define  IRQ_RX_ACTIVE       0x00
#define  IRQ_TX_ACTIVE       0x01
#define  IRQ_RSSI_VLD        0x02
#define  IRQ_PREAM_OK        0x03
#define  IRQ_SYNC_OK         0x04
#define  IRQ_NODE_OK         0x05
#define  IRQ_CRC_OK          0x06
#define  IRQ_PKT_OK          0x07
#define  IRQ_SL_TMO          0x08
#define  IRQ_RX_TMO          0x09
#define  IRQ_TX_DONE         0x0A
#define  IRQ_RX_FIFO_NMTY    0x0B
#define  IRQ_RX_FIFO_TH      0x0C
#define  IRQ_RX_FIFO_FULL    0x0D
#define  IRQ_RX_FIFO_WBYTE   0x0E
#define  IRQ_RX_FIFO_OVF     0x0F
#define  IRQ_TX_FIFO_NMTY    0x10
#define  IRQ_TX_FIFO_TH      0x11
#define  IRQ_TX_FIFO_FULL    0x12
#define  IRQ_STATE_IS_STBY   0x13
#define  IRQ_STATE_IS_FS     0x14
#define  IRQ_STATE_IS_RX     0x15
#define  IRQ_STATE_IS_TX     0x16
#define  IRQ_LED             0x17
#define  IRQ_TRX_ACTIVE      0x18
#define  IRQ_PKT_DONE        0x19

#define  SL_TMO_IRQ_EN          0x80
#define  RX_TMO_IRQ_EN          0x40
#define  TX_DONE_IRQ_EN         0x20
#define  PREAM_OK_IRQ_EN        0x10
#define  SYNC_OK_IRQ_EN         0x08
#define  NODE_OK_IRQ_EN         0x04
#define  CRC_OK_IRQ_EN          0x02
#define  PKT_DONE_IRQ_EN        0x01

#define GET_IRQ_SL_TMO   0x2000
#define GET_IRQ_RX_TMO   0x1000
#define GET_IRQ_TX_DONE  0x0800
#define GET_IRQ_LBD      0x0080
#define GET_IRQ_COL_ERR  0x0040
#define GET_IRQ_PKT_ERR  0x0020
#define GET_IRQ_PREAM_OK 0x0010
#define GET_IRQ_SYNC_OK  0x0008
#define GET_IRQ_NODE_OK  0x0004
#define GET_IRQ_CRC_OK   0x0002
#define GET_IRQ_PKT_OK   0x0001

typedef enum 
{
    GO_INIT          =0x00,        
    GO_BUSY,          
    GO_STBY,          
    GO_RECEIVE,       
    GO_WAIT_RECEIVE,          
    GO_TRANSMIT,      
    GO_WAIT_TRANSMIT, 
    GO_SLEEP,         
    GO_ERROR         
}E49x_Status_t;

static E49x_Status_t E49x_Status = GO_INIT;
static uint8e_t E49x_ReceiveBuffer[64] = {0};
static uint8e_t E49x_IRQ_Trigger = 0;
/* * 
 * ׼ 
 * ʼƵ:850MHz
 * Data Rate:2.4kbps
 * Deviation:4.8kHz
 * Bandwidth:Auto
 */
#if defined(EBYTE_E49_900M20S )
uint8e_t E49x_Config_CMT[12] = {
    0x00,0x66,0xEC,0x1D,0xF0,0x80,0x14,0x08,0x11,0x02,0x02,0x00
};
uint8e_t E49x_Config_System[12] = {
    0xAE,0xE0,0x35,0x00,0x00,0xF4,0x10,0xE2,0x42,0x20,0x00,0x81    
};
uint8e_t E49x_Config_Frequency[8] = {
    0x41,0x6D,0x80,0x86,0x41,0x62,0x27,0x16  
};
uint8e_t E49x_Config_DataRate[24] = {
    0x32,0x18,0x10,0x99,0xC1,0x9B,0x06,0x0A,0x9F,0x39,0x29,0x29,0xC0,0x51,0x2A,0x53,0x00,0x00,0xB4,0x00,0x00,0x01,0x00,0x00   
};
uint8e_t E49x_Config_Baseband[29] = {
    0x12,0x08,0x00,0xAA,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0xD4,0x2D,0x01,0x1F,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x60,0xFF,0x00,0x00,0x1F,0x10   
};
uint8e_t E49x_Config_TX[11] = {
    0x50,0x83,0x01,0x00,0x42,0xB0,0x00,0x8A,0x18,0x3F,0x7F
};
#elif defined(EBYTE_E49_400M20S )
uint8e_t E49x_Config_CMT[12] = {
0x00,
0x66,
0xEC,
0x1D,
0xF0,
0x80,
0x14,
0x08,
0x91,
0x02,
0x02,
0xD0
};
uint8e_t E49x_Config_System[12] = {
0xAE,
0xE0,
0x35,
0x00,
0x00,
0xF4,
0x10,
0xE2,
0x42,
0x20,
0x00,
0x81 
};
uint8e_t E49x_Config_Frequency[8] = {
0x3F,
0x29,
0xED,
0x11,
0x3F,
0x13,
0x3B,
0x61

};
uint8e_t E49x_Config_DataRate[24] = {
0x32,
0x18,
0x10,
0x99,
0xE1,
0x9B,
0x19,
0x0A,
0x9F,
0x39,
0x29,
0x29,
0xC0,
0x51,
0x2A,
0x53,
0x00,
0x00,
0xB4,
0x00,
0x00,
0x01,
0x00,
0x00 
};
uint8e_t E49x_Config_Baseband[29] = {
0x12,
0x08,
0x00,
0xAA,
0x02,
0x00,
0x00,
0x00,
0x00,
0x00,
0x00,
0xD4,
0x2D,
0x01,
0x1F,
0x00,
0x00,
0x00,
0x00,
0x00,
0x01,
0x00,
0x00,
0x60,
0xFF,
0x00,
0x00,
0x1F,
0x10
};
uint8e_t E49x_Config_TX[11] = {
0x50,
0x9A,
0x0C,
0x00,
0x42,
0xB0,
0x00,
0x8A,
0x18,
0x3F,
0x7F
};
#endif

/* !
 * @brief IOģ˫SPIʱ  ʱ
 * 
 * @param time 
 */
static void E49x_SpiDelay(uint8e_t time)
{
    uint8e_t n = time;
    while(n--);
}

/* !
 * @brief IOģ˫SPI 1Byte=8Bit λ 
 * 
 * @param data 
 * @note ʱοĵ AN1422 SPIдĴʱ
 */
static void E49x_HalfSpiTransmit( uint8e_t data )
{
    uint8e_t i;
  
    /*  1Byte=8bitλ */
    for( i=0 ; i<8 ; i++)
    {
        /* SLCK=0 ΪCPOL=0 ʱΪ */
        Ebyte_Port_SlckIoControl(0);
        
        /* λ0x80 ΪMSBʽ */
        if( data & 0x80 )
        {
            Ebyte_Port_SdioIoControl(1);
        }
        else
        {
            Ebyte_Port_SdioIoControl(0);
        }
        
        E49x_SpiDelay(7);
        
        /* λ */
        data <<=1;
        
        /* SLCK= 1 ΪCPOA=0 һв */
        Ebyte_Port_SlckIoControl(1);
        
        E49x_SpiDelay(7);
    }  
}

/* !
 * @brief IOģ˫SPI λȡ 8Bitλ 
 * 
 * @return ȡ 1Byte
 * @note ʱοĵ AN1422 SPIĴʱ
 */
static uint8e_t E49x_HalfSpiReceive( void )
{
    uint8e_t i;  
    uint8e_t data = 0xFF;
    
    for( i=0 ; i<8; i++ )
    {
        /* SLCK=0 ΪCPOL=0 ʱΪ */
        Ebyte_Port_SlckIoControl(0);
        
        E49x_SpiDelay(7);
        
        data <<= 1;
        
        /* SLCK=1 ӻBit */
        Ebyte_Port_SlckIoControl(1);
        
        /* ȡSDIO ¼Bit */
        if( Ebyte_Port_SdioIoControl(4))
        {
            data |= 0x01;
        }
        else
        {
            data &= ~0x01;
        }
        
        E49x_SpiDelay(7);
    }
    
    return data;
}

/* !
 * @brief ַͨڲĴд
 * 
 * @param address Ĵַ 
 * @param data  
 * @note  2߰˫SPI+2CSƬѡͨŽӿڣǱ׼4ȫ˫SPIӿ
 */
static void E49x_SetRegister( uint8e_t address, uint8e_t data )
{

    /* SDIO */
    Ebyte_Port_SdioIoControl(3);//SDIOΪģʽ
    Ebyte_Port_SdioIoControl(1);//SDIOߵƽ  
    
    /* SLCK  */
    Ebyte_Port_SlckIoControl(0);
    
    /* CS    */
    Ebyte_Port_FcsbIoControl(1);
    Ebyte_Port_CsbIoControl(0); //CSѡһ CSB͵ƽʾѡĴ
    
    /* ٵȴ SLCK */
    E49x_SpiDelay(20);
    
    /* дַ */
    E49x_HalfSpiTransmit( address & 0x7F );//ַ⴦ 0x7F r/wλ=0
    
    /* д */
    E49x_HalfSpiTransmit( data ); 
    
    /* ͷ */
    Ebyte_Port_SlckIoControl(0);
    E49x_SpiDelay(20);
    Ebyte_Port_CsbIoControl(1);
    
    /* SDIO */
    Ebyte_Port_SdioIoControl(2); //ĬлΪ 
}

/* !
 * @brief ַͨڲĴȡ
 * 
 * @param address Ĵַ 
 * @return data  1Byte 
 * @note  2߰˫SPI+2CSƬѡͨŽӿڣǱ׼4ȫ˫SPIӿ
 *        ʱοĵ AN1422
 */
static uint8e_t E49x_GetRegister( uint8e_t address )
{
    uint8e_t result = 0xFF;

    /* SDIO  */
    Ebyte_Port_SdioIoControl(3);//SDIOΪģʽ
    Ebyte_Port_SdioIoControl(1);//SDIOߵƽ  

    /* SLCK  */
    Ebyte_Port_SlckIoControl(0);
    
    /* CS    */
    Ebyte_Port_FcsbIoControl(1);
    Ebyte_Port_CsbIoControl(0); //CSѡһ CSB͵ƽʾѡĴ
    
    /* ٵȴ SLCK */
    E49x_SpiDelay(20);
    
    /* дַ */
    E49x_HalfSpiTransmit( address|0x80 ); //ַ⴦ 0x80 r/wλ=1
    
    /* SDIO лΪ  */
    Ebyte_Port_SdioIoControl(2);
    
    /*  */
    result = E49x_HalfSpiReceive();
    
    /* ͷ */
    Ebyte_Port_SlckIoControl(0);   
    E49x_SpiDelay(20);   
    Ebyte_Port_CsbIoControl(1);
    
    return result;
}

/* !
 * @brief дڲݻ FIFO
 * 
 * @param data ָд׵ַָ
 * @param size дݳ
 * @note һ֧32ֽڡ˺ϲͺĶFIFO֧64ֽ
 */
static void E49x_SetFIFO( uint8e_t *data , uint16e_t size )
{
    uint8e_t i;
  
    /* CS Ÿλ */
    Ebyte_Port_FcsbIoControl(1);
    Ebyte_Port_CsbIoControl(1);
    
    /* SLCK  */
    Ebyte_Port_SlckIoControl(0);
    
    /* SDIO лΪ  */
    Ebyte_Port_SdioIoControl(3);
    
    /* ѭд  */
    for( i=0 ; i<size ; i++)
    {
        /* CSѡһ FCSB͵ƽʾѡڲFIFO */
        Ebyte_Port_FcsbIoControl(0); 
        
        E49x_SpiDelay(20);
        
        /* д */
        E49x_HalfSpiTransmit( data[i] );
        
        Ebyte_Port_SlckIoControl(0);
        
        E49x_SpiDelay(30);
        
        Ebyte_Port_FcsbIoControl(1);
        
        E49x_SpiDelay(60);
    }
    
    Ebyte_Port_SlckIoControl(0);
}

/* !
 * @brief ڲFIFO
 * 
 * @param data ָ׵ַָ
 * @param size ȡݳ
 * @return 0: 1:FIFOݵȱʾλȷ
 * @note һFIFOΪ32ֽ(1ֽڱȱʾλռ)˺ϲͺĶFIFO֧64ֽ
 */
static uint8e_t E49x_GetFIFO( uint8e_t *data , uint8e_t *size )
{
    uint8e_t i,length;
    
    /* CS Ÿλ */
    Ebyte_Port_FcsbIoControl(1);
    Ebyte_Port_CsbIoControl(1);
    
    /* SLCK  */
    Ebyte_Port_SlckIoControl(0);
  
    /* SDIO лΪ  */
    Ebyte_Port_SdioIoControl(2);
    
    /* ȶȡһֽڳȱʾλ 
       Ϊ˿ɱ䳤壬λE49ͷӲԶ  
    */
    Ebyte_Port_FcsbIoControl(0);   
    E49x_SpiDelay(20);    
    length = E49x_HalfSpiReceive();    
    Ebyte_Port_SlckIoControl(0);    
    E49x_SpiDelay(30);    
    Ebyte_Port_FcsbIoControl(1);    
    E49x_SpiDelay(60);    
    
    /* жϳȱʾǷȷ */
    if( length > 64 ) return 1;
    length -= 1;//ȥ1ȱʾλ ݳ
    
    /* ѭȡ  */
    for( i=0 ; i<length ; i++)
    {
        /* CSѡһ FCSB͵ƽʾѡڲFIFO */
        Ebyte_Port_FcsbIoControl(0);
        
        E49x_SpiDelay(20);
        
        /*  */
        data[i] = E49x_HalfSpiReceive();
        
        Ebyte_Port_SlckIoControl(0);
        
        E49x_SpiDelay(30);
        
        Ebyte_Port_FcsbIoControl(1);
        
        E49x_SpiDelay(60);
    } 
    
    /* 䳤 */
    *size = length;
    
    return 0;
}

/* !
 * @brief FIFO
 * @param mode ģʽ
 *           @arg FIFO_WRITE дģʽ
 *           @arg FIFO_READ  ģʽ
 *
 */
static void E49x_ClearFIFO( uint8e_t mode)
{
    if( mode == FIFO_WRITE )
    {    
        /* FIFO Ĵַ:0x6C bit0:0x01  */
        E49x_SetRegister(0x6C, 0x01);//bit0  0Ч1 TX FIFO λ 1 ֮轫 0 ڲԶΪ0
    }
    else
    {
        /* FIFO Ĵַ:0x6C bit1:0x02 */
        E49x_SetRegister(0x6C, 0x02);//bit1  0Ч1 RX FIFO λ 1 ֮轫 0 ڲԶΪ0     
    } 
}

/* !
 * @brief FIFOдģʽ 
 * @param mode ģʽ
 *           @arg FIFO_WRITE дģʽ
 *           @arg FIFO_READ  ģʽ
 *
 */
static void E49x_GoFIFO( uint8e_t mode)
{
    uint8e_t config;
    
    /* Ĵַ:0x69  bit2:0x04  bit0:0x01 */
    config = E49x_GetRegister(0x69);
    
    if( mode == FIFO_WRITE )
    {    
        config |= 0x04;//bit2 Ϊһ 64-byte FIFO ʱʹá0 RX FIFO1 TX FIFO
        config |= 0x01;//bit0 ڷ FIFO ǰúá0SPI ĲǶ FIFO1SPI Ĳд FIFO
    }
    else
    {
        config &= ~0x04;//bit2 
        config &= ~0x01;//bit0
    }
    
    E49x_SetRegister(0x69, config);  
}

/* !
 * @brief λ 
 * @note ģڲõĲᶪʧ
 */
void E49x_Reset(void)
{
    /* ָλ */
    E49x_SetRegister(0x7F, 0xFF);
}

/* !
 * @brief дĴ Ϊģʽ 
 *
 * @return 0: 1:쳣
 * @note ģʽ²üĴ
 */
static uint8e_t E49x_SetStby(void)
{
    uint8e_t retry  =10;
    uint8e_t result =1 ;
    
    
    /* ״̬:׼״̬л */
    E49x_Status = GO_BUSY;
    
    /* дָģʽ Ĵַ:0x60  bit1:0x02 */
    E49x_SetRegister(0x60,0x02);
    

    while( retry > 0)
    {
        Ebyte_Port_DelayUs(100);
        
        /* ȡĴȷ Ĵַ:0x61 bit3bit0=0x02 */
        result = E49x_GetRegister(0x61) & 0x0F;
        if( result == 0x02 )
        {
            /* ״̬¼ */
            E49x_Status = GO_STBY;
            
            /*  */
            result = 0;
            break;
        }    
        
        Ebyte_Port_DelayUs(500);
        retry--;
    }
    
    /*  */
    if( result!=0 ) 
    {
        /* ״̬:ģ쳣 */
        E49x_Status = GO_ERROR;        
    }
    
    return result;
}



/* !
 * @brief дĴ Ϊģʽ 
 *
 * @return 0: 1:쳣
 */
static uint8e_t E49x_SetReceive(void)
{
    uint8e_t retry  =10;
    uint8e_t result =1 ;
    
    /* ״̬:׼״̬л */
    E49x_Status = GO_BUSY;    
    
    /* дָģʽ Ĵַ:0x60  bit3:0x08 */
    E49x_SetRegister(0x60,0x08);
    

    while( retry > 0)
    {
        Ebyte_Port_DelayUs(100);
        
        /* ȡĴȷ Ĵַ:0x61 bit3bit0=0x05 */
        result = E49x_GetRegister(0x61) & 0x0F;
        if( result == 0x05 )
        {
            /* ״̬¼ */
            E49x_Status = GO_RECEIVE;
            
            /*  */
            result = 0;
            break;
        }    
        
        Ebyte_Port_DelayUs(100);
        
        /* ȡĴȷ Ĵַ:0x6D 1 bitΪЧλ*/
        result = E49x_GetRegister(0x6D) & 0x01;
        if( result == 0x01 )
        {
            /* ״̬¼ */
            E49x_Status = GO_RECEIVE;
            
            /*  */
            result = 0;
            break;
        }
        
        Ebyte_Port_DelayUs(500);
        retry--;
    }
    

    /*  */
    if( result!=0 ) 
    {
        /* ״̬:ģ쳣 */
        E49x_Status = GO_ERROR;        
    }
    
    return result;
}

/* !
 * @brief дĴ Ϊģʽ 
 *
 * @return 0: 1:쳣
 * @note ԶĶFIFOеݽߴ
 */
static uint8e_t E49x_SetTransmit(void)
{
    uint8e_t retry  =10;
    uint8e_t result =1 ;
    
    /* ״̬:׼״̬л */
    E49x_Status = GO_BUSY;
    
    /* дָ뷢ģʽ Ĵַ:0x60  bit6:0x40 */
    E49x_SetRegister(0x60,0x40);
    
    while( retry > 0)
    {
        Ebyte_Port_DelayUs(100);
        
        /* ȡĴȷ Ĵַ:0x61 bit3bit0=0x06*/
        result = E49x_GetRegister(0x61) & 0x0F;
        if( result == 0x06 )
        {
            /* ״̬¼ */
            E49x_Status = GO_TRANSMIT;
            
            /*  */
            result = 0;
            break;
        }    
        
        Ebyte_Port_DelayUs(100);
        
        /* ȡĴȷ Ĵַ:0x6A  bit3=0x08 */
        result = E49x_GetRegister(0x6A) & 0x08;
        if( result == 0x08 )
        {
            /* ״̬¼ */
            E49x_Status = GO_TRANSMIT;
            
            /*  */
            result = 0;
            break;
        }  

        Ebyte_Port_DelayUs(500); 
        retry--;
    }
    
    /*  */
    if( result!=0 ) 
    {
        /* ״̬:ģ쳣 */
        E49x_Status = GO_ERROR;        
    }
    
    return result;
}

/* !
 * @brief дĴ Ϊģʽ 
 *
 * @return 0: 1:쳣
 */
static uint8e_t E49x_SetSleep(void)
{
    uint8e_t retry  =10;
    uint8e_t result =1 ;
    
    /* ״̬:׼״̬л */
    E49x_Status = GO_BUSY;
    
    /* дָģʽ Ĵַ:0x60  bit4:0x10 */
    E49x_SetRegister(0x60,0x10);
    
    while( retry > 0)
    {
        Ebyte_Port_DelayUs(100);
        
        /* ȡĴȷ Ĵַ:0x61  bi3bit1=0x01*/
        result = E49x_GetRegister(0x61) & 0x0F;
        if( result == 0x01 )
        {
            /* ״̬¼ */
            E49x_Status = GO_SLEEP;
            
            /*  */
            result = 0;
            break;
        }    
        
        Ebyte_Port_DelayUs(500);
        retry--;
    }
    
    /*  */
    if( result!=0 ) 
    {
        /* ״̬:ģ쳣 */
        E49x_Status = GO_ERROR;        
    }
    
    return result;
}


/* !
 * @brief ʵʷϢĳ
 * 
 * @param size ĳ
 *
 * @note ĶNODEID οAN143ֲȷϢ
 *        : Ĵ0x47 bit3:2 NODEID_SIZEΪ1 NODEIDΪ2
 *              ôӦϢ峤Ϊ ʾλ1+NODEID2+ûݳ = ûݳ+3
 */
static void E49x_SetPayloadSize( uint16e_t size )
{
    uint8e_t config; 
      
    /* Ĵַ:0x45 bit6bit4:0x70  Ӧ峤10:8bit*/  
    config = E49x_GetRegister(0x45);

    config &= ~0x70;
    config |= (size >> 4) & 0x70;
    E49x_SetRegister(0x45, config);
    
    /* Ĵַ:0x46 bit7bit0:0xFF Ӧ峤7:0bit*/
    config = size & 0xFF;
    E49x_SetRegister(0x46, config);      
}

/* !
 * @brief ͨIOʹ÷ʽ
 * 
 * @param config 궨  GPIO1_xxx | GPIO2_xxx | GPIO3_xxx
 *
 * @note һ㶼ʹIOΪжԴ ֪ͨMCU/
 */
static void E49x_SetGPIO( uint8e_t config )
{
    /* Ĵַ:0x65 */
    E49x_SetRegister(0x65, config);
}

/* !
 * @brief ж
 * 
 * @param configIrq1 ж1 궨 
 * @param configIrq2 ж2 궨 
 *
 * @note һ㶼÷/жϣE49x_SetGPIOӳ䵽IO
 */
static void E49x_SetIRQ(uint8e_t configIrq1, uint8e_t configIrq2)
{
    
    /* ж1ӳ ռbit4bit00x1F Ĵַ:0x66 */
    configIrq1 &= 0x1F;
    configIrq1 |= (~0x1F) & E49x_GetRegister(0x66);
    E49x_SetRegister(0x66, configIrq1);

    /* ж2ӳ ռbit4bit00x1F Ĵַ:0x67 */
    configIrq2 &= 0x1F;
    configIrq2 |= (~0x1F) & E49x_GetRegister(0x67);
    E49x_SetRegister(0x67, configIrq2);
}



/* !
 * @brief ȡжϱ־
 *
 * @return  жλbit
 *  
 *           0x2000:SL_TMO ˯߳ʱжϱ־
 *           0x1000:RX_TMO ճʱжϱ־
 *           0x0800:TX_DONE жϱ־   
 *           0x0080:LBD Чɹ⵽͵ѹжϱ־
 *           0x0040:COL_ERR жϱ־
 *           0x0020:PKT_ERR жϱ־
 *           0x0010:PREAM_OK ǰɹжϱ־
 *           0x0008:SYNC_OK ּͬɹжϱ־
 *           0x0004:NODE_OK ڵIDɹжϱ־
 *           0x0002:CRC_OK  CRCɹжϱ־
 *           0x0001:PKT_OK  ݰɣܶԴжʹжϱ־
 *
 */
static uint16e_t E49x_GetIRQ(void)
{
    uint8e_t irqPolar,irqSetFlag6D,irqSetFlag6A;
    
    /* ȡжϼĴ Ĵַ:0x6D + 0x6A */
    irqSetFlag6D = E49x_GetRegister(0x6D);
    irqSetFlag6A = E49x_GetRegister(0x6A);
    
    /* ȡжϼ(0:ʾߵƽЧж 1:ʾ͵ƽЧж) bit1:0x20 Ĵַ:0x66 */
    irqPolar = (E49x_GetRegister(0x66) & 0x20)? 1:0 ;
    
    /* ͵ƽΪЧжʱ Ҫλ  */
    if( irqPolar )
    {
        irqSetFlag6D = ~irqSetFlag6D;
        irqSetFlag6A = ~irqSetFlag6A;
    }    
    
    return  (irqSetFlag6A<<8) | irqSetFlag6D;
}

/* !
 * @brief жϱ־
 *
 * @note  жλbit
 *           Ĵַ 0x6D
 *           bit7:LBD Чɹ⵽͵ѹжϱ־
 *           bit6:COL_ERR жϱ־
 *           bit5:PKT_ERR жϱ־
 *           bit4:PREAM_OK жϱ־
 *           bit3:SYNC_OK жϱ־
 *           bit2:NODE_OK жϱ־
 *           bit1:CRC_OK жϱ־
 *           bit0:PKT_OK жϱ־
 *
 *           Ĵַ 0x6A
 *           bit5:SL_TMO жϱ־
 *           bit4:RX_TMO жϱ־
 *           bit3:TX_DONE жϱ־
 *
 * @note  жbit
 *           Ĵַ 0x6A
 *           bit2:TX_DONE ж
 *           bit1:SL_TMO ж
 *           bit0:RX_TMO ж
 *
 *           Ĵַ 0x6B
 *           bit5:LBD Чɹ⵽͵ѹж
 *           bit4:PREAM_OK ж
 *           bit3:SYNC_OK ж
 *           bit2:NODE_OK ж
 *           bit1:CRC_OK ж
 *           bit0:PKT_DONE ж
 */
void E49x_ClearIRQ(void)
{
    uint8e_t irqPolar,irqSetFlag6D,irqSetFlag6A;
    uint8e_t irqClearFlag6A = 0;
    uint8e_t irqClearFlag6B = 0;
    
    /* ȡжϼĴ Ĵַ:0x6D + 0x6A */
    irqSetFlag6D = E49x_GetRegister(0x6D);
    irqSetFlag6A = E49x_GetRegister(0x6A);
    
    /* ȡжϼ(0:ʾߵƽЧж 1:ʾ͵ƽЧж) bit1:0x20 Ĵַ:0x66 */
    irqPolar = (E49x_GetRegister(0x66) & 0x20)? 1:0 ;
    
    /* ͵ƽΪЧжʱ Ҫλ  */
    if( irqPolar )
    {
        irqSetFlag6D = ~irqSetFlag6D;
        irqSetFlag6A = ~irqSetFlag6A;
    }
    
    /* жλʶλʶλһ Ҫλ */
    if( irqSetFlag6D & 0x80 ) 
    {
        irqClearFlag6B |= 0x20;  //Ĵ0x6D-LBDλbit7 Ӧ Ĵ0x6B-LBDbit5
    }  
    if( irqSetFlag6D & 0x40 ) 
    {
        irqClearFlag6B |= 0x01;  //ĴCOL_ERRλbit6  Ӧ Ĵ0x6B-PKT_DONEbit0
    }        
    if( irqSetFlag6D & 0x20 ) 
    {
        irqClearFlag6B |= 0x01;  //Ĵ0x6D-PKT_ERRλbit5 Ӧ Ĵ0x6B-PKT_DONEbit0
    }    
    if( irqSetFlag6D & 0x10 ) 
    {
        irqClearFlag6B |= 0x10;  //Ĵ0x6D-PREAM_OKλbit4 Ӧ Ĵ0x6B-PREAM_OKbit4
    }  
    if( irqSetFlag6D & 0x08 ) 
    {
        irqClearFlag6B |= 0x08;  //Ĵ0x6D-SYNC_OKλbit3 Ӧ Ĵ0x6B-SYNC_OKbit3
    }  
    if( irqSetFlag6D & 0x04 ) 
    {
        irqClearFlag6B |= 0x04;  //Ĵ0x6D-NODE_OKλbit2 Ӧ Ĵ0x6B-NODE_OKbit2
    }  
    if( irqSetFlag6D & 0x02 ) 
    {
        irqClearFlag6B |= 0x02;  //Ĵ0x6D-CRC_OKλbit1 Ӧ Ĵ0x6B-CRC_OKbit1
    }  
    if( irqSetFlag6D & 0x01 ) 
    {
        irqClearFlag6B |= 0x01;  //Ĵ0x6D-PKT_OKλbit0 Ӧ Ĵ0x6B-PKT_DONEbit0
    }      
    
    if( irqSetFlag6A & 0x20 ) 
    {
        irqClearFlag6A |= 0x02;  //Ĵ0x6A-SL_TMOλbit5 Ӧ Ĵ0x6A-SL_TMObit1
    }  
    if( irqSetFlag6A & 0x10 ) 
    {
        irqClearFlag6A |= 0x01;  //Ĵ0x6A-RX_TMOλbit4 Ӧ Ĵ0x6A-PKT_DONEbit0
    }  
    if( irqSetFlag6A & 0x08 ) 
    {
        irqClearFlag6A |= 0x04;  //Ĵ0x6A-TX_DONEλbit3 Ӧ Ĵ0x6A-TX_DONEbit2
    }      
    
    /* ʶдؼĴ */
    E49x_SetRegister(0x6A, irqClearFlag6A);
    E49x_SetRegister(0x6B, irqClearFlag6B);
}

/* !
 * @brief ж
 * 
 * @param enableIrq жϵbitλ
 * @note һ㶼ӳӦжλ
 */
static void E49x_GoIRQ( uint8e_t  enableIrq)
{
    /* ж Ĵַ:0x68 */
    E49x_SetRegister(0x68, enableIrq);
}

/* !
 * @brief ʼ
 * 
 * @note һյ ᴥGPIOж
 */
uint8e_t E49x_GoReceive(void)
{
    uint8e_t result = 0;
    uint8e_t irqGpio = GPIO1_IRQ2;
    
    /* ģʽ */
    result = E49x_SetStby();
    if( result!=0  ) return 1;
    
    /* жͨIRQ2 */
    switch( RECEIVE_IRQ_GPIO )
    {
        case 1:irqGpio=GPIO1_IRQ2;break;
        case 2:irqGpio=GPIO2_IRQ2;break;
        case 3:irqGpio=GPIO3_IRQ2;break;
        default: break;
    }
    
    /* GPIOжӳ */                  
    E49x_SetGPIO(  irqGpio  );    
    
    /* жӳ  ｫжӳ䵽IRQ2  IRQ_TX_DONEռλ */
    E49x_SetIRQ (  IRQ_TX_DONE,  IRQ_PKT_OK );   
   
    /* òЧ */
    result = E49x_SetSleep();    
    if( result!=0  ) return 1;   
       
    /* ģʽ */
    result = E49x_SetStby();
    if( result!=0  ) return 1;
    
    /* жϱʶ */
    E49x_ClearIRQ();
      
    /* ж(ڲĴʶλ) */
    E49x_GoIRQ( PREAM_OK_IRQ_EN | SYNC_OK_IRQ_EN | PKT_DONE_IRQ_EN );
    
    /* ȡڲFIFO */
    E49x_GoFIFO( FIFO_READ );
    
    /* FIFO */
    E49x_ClearFIFO( FIFO_READ );
    
    /* ģʽ */
    result = E49x_SetReceive();
    if( result!=0  ) return 1;
    
    /* ״̬:ȴ */
    E49x_Status = GO_WAIT_RECEIVE;
     
    
    /*  0 */
    return result;
  
}

/* !
 * @brief ʼ
 * 
 * @param data ָݵָ
 * @param size ͳ
 * @return 0: 1:ʧ
 * @note עsize ϲ˷ͽFIFO ֧64ֽڣΪ32ֽ
 *       FIFO ǿյģ뷢״̬ôͻһֱ趨prefix(ǰ)ֱFIFO
 *       ÿηֶлΪģʽ
 */
uint8e_t E49x_GoTransmit( uint8e_t *data, uint8e_t size )
{
    uint8e_t result = 0;
    uint8e_t irqGpio = GPIO1_IRQ2;
    
    /* ģʽ */
    result = E49x_SetStby();
    if( result!=0  ) return 1;
    
    /* жͨIRQ2 */
    switch( TRANSMIT_IRQ_GPIO )
    {
        case 1:irqGpio=GPIO1_IRQ2;break;
        case 2:irqGpio=GPIO2_IRQ2;break;
        case 3:irqGpio=GPIO3_IRQ2;break;
        default: break;
    }    
    
    /* GPIOжӳ䵽IRQ2 */                  
    E49x_SetGPIO(  irqGpio );    
    
    /* жӳ  ｫжӳ䵽IRQ2 IRQ_PKT_OKռλ */
    E49x_SetIRQ (  IRQ_PKT_OK , IRQ_TX_DONE  );   
    
    /* òЧ */
    result = E49x_SetSleep();    
    if( result!=0  ) return 1;   
       
    /* ģʽ */
    result = E49x_SetStby();
    if( result!=0  ) return 1;
    
    /* жϱʶ */
    E49x_ClearIRQ();
      
    /* ж */
    E49x_GoIRQ( TX_DONE_IRQ_EN  );
    
    /* дڲFIFO */
    E49x_GoFIFO( FIFO_WRITE );
    
    /* FIFO */
    E49x_ClearFIFO( FIFO_WRITE );
    
    /* дFIFO */
    E49x_SetFIFO( data , size);
    
    /* ĬΪ ɱ䳤+NODEID=ݳsize+ȱʾλ1  οĵAN143 */
    E49x_SetPayloadSize( size+1 );
      
    /* 뷢ģʽ ԶFIFO*/
    result = E49x_SetTransmit();
    if( result!=0  ) return 1;
    
    /* ״̬:ȴ */
    E49x_Status = GO_WAIT_TRANSMIT;
    
    /* ģʽ ڴ˼鵽ɲŷ*/    
#if EBYTE_TRANSMIT_MODE_BLOCKED
    uint16e_t irqSet = 0;
    do
    {
        irqSet = E49x_GetIRQ();
        Ebyte_Port_DelayUs(500);
    }
    while( !(irqSet & GET_IRQ_TX_DONE ) );
    
    E49x_ClearIRQ();
    
    /* ״̬: ״̬ */
    E49x_SetStby();
    
    /*  صûӿں */
    Ebyte_Port_TransmitCallback( irqSet );
#endif    
    /*  0 */
    return result;    
}

/* !
 * @brief ģǷд
 *
 * @return 0: 1:д,Ӳ
 */
static uint8e_t E49x_IsAlive(void)
{
    uint8e_t back, getData ;
    uint8e_t testRegister = 0x48;
    uint8e_t testData = 0xAA;
    uint8e_t result = 1;
      
    /* Ĵ0x48д test */
    back = E49x_GetRegister(testRegister);
    E49x_SetRegister(testRegister, testData);

    /* ȡȷ ָԭ */
    getData = E49x_GetRegister(testRegister);
    E49x_SetRegister(testRegister, back);  
    
    if( getData == testData )
    {
        /*  */
        result = 0;
    }

    return result;
}

/* !
 * @brief Ʋ 
 */
static void E49x_Config(void)
{
    uint8e_t config = 0;  
    uint8e_t i,start;
    
    /* ñ bit4:0x10, رոλ bit5:0x20 Ĵַ:0x61 */
    config  = E49x_GetRegister(0x61);
    config |= 0x10;       
    config &= ~0x20;      
    E49x_SetRegister(0x61, config);

    /* ໷Ƶ (һƵʲȷ޷) bit5:0x20 Ĵַ:0x62 */
    config  = E49x_GetRegister(0x62);
    config |= 0x20;        
    E49x_SetRegister(0x62, config);
    
    /* رLFOSCģ LFOSCҪ Sleep Timerرպ󽵵߹ԼԼ5msĽʱ䣩bit7bit5=0 Ĵַ:0x0D  */
    config  = E49x_GetRegister(0x0D);
    config &= ~0xE0;
    
    /* ϲ/ոԶ32ֽFIFO 64ֽFIFO bit1:0x02  Ĵַ:0x69 */
    config  = E49x_GetRegister(0x69);
    config |= 0x02; 
    E49x_SetRegister(0x69, config);
    
    /* ɵĲ д뵽Ĵ */
    /* CMT  ʼַ:0x00 :12 */
    start = 0;
    for( i=0 ; i<12; i++ )
    {
        E49x_SetRegister( start+i, E49x_Config_CMT[i] );
    }
    /* ϵͳ   ʼַ:0x0C :12 */
    start = 0x0C;
    for( i=0 ; i<12; i++ )
    {
        E49x_SetRegister( start+i, E49x_Config_System[i] );
    }    
    /* Ƶ   ʼַ:0x18 :8 */
    start = 0x18;
    for( i=0 ; i<8; i++ )
    {
        E49x_SetRegister( start+i, E49x_Config_Frequency[i] );
    }       
    /*  ʼַ:0x20 :24 */
    start = 0x20;
    for( i=0 ; i<24; i++ )
    {
        E49x_SetRegister( start+i, E49x_Config_DataRate[i] );
    }   
    /*    ʼַ:0x38 :29 */
    start = 0x38;
    for( i=0 ; i<29; i++ )
    {
        E49x_SetRegister( start+i, E49x_Config_Baseband[i] );
    } 
    /*    ʼַ:0x55 :11 */
    start = 0x55;
    for( i=0 ; i<11; i++ )
    {
        E49x_SetRegister( start+i, E49x_Config_TX[i] );
    } 
       
    /* ԭBUGҪ 1.4汾ԺԴ */
    config = (~0x07) & E49x_GetRegister(0x09);
    E49x_SetRegister(0x09, config|0x02);   
    
    /* Ƶʲ׼ Ĵַ:0x64 㹫ʽΪ 2.5KHz*n */
    E49x_SetRegister(0x64, EBYTE_E49x_FREQUENCY_STEP );
    
    /* Ƶʲۼ Ĵַ:0x63 㹫ʽΪ ׼*n */
    E49x_SetRegister(0x63, EBYTE_E49x_FREQUENCY_CHANNEL );
    
    /* òͨģʽлЧ */
    E49x_SetSleep();
    
}

/* !
 * @brief ģģʽ
 *
 * @return 0: 1:ʧ
 */
uint8e_t E49x_GoSleep(void)
{
    return E49x_SetSleep();
}

/* !
 * @brief ʼ
 *
 * @return 0: 1:ʼʧ
 */
uint8e_t E49x_Init(void)
{
    uint8e_t i;
    uint8e_t result = 0 ;
  
    /* λ ᶪʧ */
    E49x_Reset();
    
    /* ٵȴ20ms */
    for (i=0;i<20;i++)
    {
        Ebyte_Port_DelayUs(1000);
    }    
    
    /* ģʽ */
    result = E49x_SetStby();
    if( result!=0  ) return 1;
    
    /* Ĵд */
    result = E49x_IsAlive();
    if( result!=0  ) return 1;
    
    /* õƲ */
    E49x_Config();    
    
    /* ĬϿʼ */
    result = E49x_GoReceive();
    if( result!=0  ) return 1;
    
    return result;
}

/* !
 * @brief ûڵú ɽջص
 *
 * @return 0: 1:ݲȷ
 */
uint8e_t E49x_TaskForPoll(void)
{
     uint16e_t irqSet = 0;
     uint8e_t getSize = 0;
     
    /* ״̬ ȴжź */
    if( E49x_Status == GO_WAIT_RECEIVE  )
    {       
        /* ȡ״̬ */
        irqSet = E49x_GetIRQ();

        /* Ƿݽ(жϱʶλ) */
        if( irqSet & GET_IRQ_PKT_OK )
        {          
            /* ģ */
            E49x_SetStby();
            
            /* ȡ */
            E49x_GetFIFO( E49x_ReceiveBuffer, &getSize );
            
            /* ж */
            E49x_ClearIRQ();
            
            /* صɺ */
            Ebyte_Port_ReceiveCallback( GET_IRQ_PKT_OK , E49x_ReceiveBuffer ,getSize);
        }
        else
        {
            //to-do ж ûԼ
//            Ebyte_Port_ReceiveCallback( irqSet, 0 , 0 );
        }
        
        E49x_ClearIRQ();
        
        /* λ */
        E49x_IRQ_Trigger = 0;        
    }
    
    /* ģʽ ״̬ ȴжź */
#if !(EBYTE_TRANSMIT_MODE_BLOCKED)    
    if( E49x_Status == GO_WAIT_TRANSMIT && E49x_IRQ_Trigger  )
    {
        /* ȡ״̬ */
        irqSet = E49x_GetIRQ();

        /* Ƿ(жϱʶλ) */
        if( irqSet & GET_IRQ_TX_DONE )
        {
            /* ģ */
            E49x_SetStby();       
            
            /* ж */
            E49x_ClearIRQ();
            
            /*  صûӿں */
            Ebyte_Port_TransmitCallback( irqSet );
        } 
        
        /* λ */
        E49x_IRQ_Trigger = 0;
    }
#endif    
    
    return 0;
}

/* !
 * @brief жϴ 
 * @note  һ˵IOже ѭTaskForPollź
 */
void E49x_TaskForIRQ(void)
{
    E49x_IRQ_Trigger = 1;
}

/*!
 * @brief ȡģ
 * 
 * @return ַָָ 
 * @note ׼ַ ĩβн '\0'
 */
uint8e_t* E49x_GetName(void)
{
    return E49x_NameString;
}

/*!
 * @brief ȡģ״̬
 * 
 * @return ״̬
 *        δʼ     GO_INIT          =0x00        
 *        л״̬ GO_BUSY          =0x01   
 *        /    GO_STBY          =0x02   
 *        ׼ջ GO_RECEIVE       =0x03   
 *        ڼ GO_WAIT_RECEIVE  =0x04          
 *        ׼ͻ GO_TRANSMIT      =0x05   
 *        ȴ GO_WAIT_TRANSMIT =0x06   
 *                 GO_SLEEP         =0x07   
 *        ڲ     GO_ERROR         =0x08 
 */
uint8e_t E49x_GetStatus(void)
{
  return (uint8e_t)E49x_Status;;
}

/* !
 * @brief ȡģ汾
 * 
 * @return 8λı 
 * @note 0x10 V1.0
 */
uint8e_t E49x_GetDriverVersion(void)
{
  return  EBYTE_E49_PROGRAM_TYPE;
}
